mutable struct EnsInfo
    id::String
    L::Int64
    beta::Float64
    deg::Bool
    mpi::Float64
    dtr::Int64
    trunc::Union{Int64, Vector{Int64}, Nothing}
    function EnsInfo(ens_id::String, info::Vector{Float64})
        id = ens_id
        L = info[1]
        beta = info[2]
        deg = info[3]
        mpi = info[4]
        dtr = info[5]
        return new(id, L, beta, deg, mpi, dtr, nothing)
    end
    function EnsInfo(ens_id::String, info::Vector{Float64}, trunc::Union{Nothing, Int64, Vector{Int64}})
        id = ens_id
        L = info[1]
        beta = info[2]
        deg = info[3]
        mpi = info[4]
        dtr = info[5]
        return new(id, L, beta, deg, mpi, dtr, trunc)
    end
end
mutable struct EnsObs
    ensinfo::EnsInfo
    mu_list:: Vector{Vector{Float64}}
    is_pseudo::Bool

    m_ll::Union{Nothing,uwreal}
    m_ls::Union{Nothing,uwreal}
    m_lh::Union{Nothing,Vector{uwreal}}
    m_ss::Union{Nothing,uwreal}
    m_sh::Union{Nothing,Vector{uwreal}}
    m_hh::Union{Nothing,Vector{uwreal}}
    f_ll::Union{Nothing,uwreal}
    f_ls::Union{Nothing,uwreal}
    f_lh::Union{Nothing,Vector{uwreal}}
    f_ss::Union{Nothing,uwreal}
    f_sh::Union{Nothing,Vector{uwreal}}
    f_hh::Union{Nothing,Vector{uwreal}}

    m_ll_vec::Union{Nothing,uwreal}
    m_ls_vec::Union{Nothing,uwreal}
    m_lh_vec::Union{Nothing,Vector{uwreal}}
    m_ss_vec::Union{Nothing,uwreal}
    m_sh_vec::Union{Nothing,Vector{uwreal}}
    m_hh_vec::Union{Nothing,Vector{uwreal}}
    f_ll_vec::Union{Nothing,uwreal}
    f_ls_vec::Union{Nothing,uwreal}
    f_lh_vec::Union{Nothing,Vector{uwreal}}
    f_ss_vec::Union{Nothing,uwreal}
    f_sh_vec::Union{Nothing,Vector{uwreal}}
    f_hh_vec::Union{Nothing,Vector{uwreal}}

    m_lh_match::Union{Nothing,uwreal}
    m_sh_match::Union{Nothing,uwreal}
    m_hh_match::Union{Nothing,uwreal}
    f_lh_match::Union{Nothing,uwreal}
    f_sh_match::Union{Nothing,uwreal}
    f_hh_match::Union{Nothing,uwreal}

    m_lh_vec_match::Union{Nothing,uwreal}
    m_sh_vec_match::Union{Nothing,uwreal}
    m_hh_vec_match::Union{Nothing,uwreal}
    f_lh_vec_match::Union{Nothing,uwreal}
    f_sh_vec_match::Union{Nothing,uwreal}
    f_hh_vec_match::Union{Nothing,uwreal}

    muh_target::Union{Nothing,uwreal}
    a::Union{Nothing,uwreal}
    t0::Union{Nothing,uwreal}
    deltam::Union{Nothing, uwreal}

    
    function EnsObs(ens::EnsInfo, mu::Vector{Vector{Float64}}, m_ps::Vector{uwreal}, f_ps::Vector{uwreal}, m_vec::Vector{uwreal}, f_vec::Vector{uwreal})
        a = new()
        a.ensinfo = ens
        a.mu_list  = mu
        a.is_pseudo = true

        a.m_ll = get_ll(a.mu_list, m_ps,a.ensinfo.deg)
        a.ensinfo.deg ? a.m_ls = a.m_ll : a.m_ls = get_ls(a.mu_list, m_ps, a.ensinfo.deg)
        a.ensinfo.deg ? a.m_ss = a.m_ll : a.m_ss = get_ss(a.mu_list, m_ps, a.ensinfo.deg)
        a.m_lh = get_lh(a.mu_list, m_ps,a.ensinfo.deg)
        a.ensinfo.deg ? a.m_sh = a.m_lh : a.m_sh = get_sh(a.mu_list, m_ps, a.ensinfo.deg)
        a.m_hh = get_hh(a.mu_list, m_ps,a.ensinfo.deg)
        a.f_ll = get_ll(a.mu_list, f_ps, a.ensinfo.deg)
        a.ensinfo.deg ? a.f_ls = a.f_ll : a.f_ls = get_ls(a.mu_list, f_ps, a.ensinfo.deg)
        a.ensinfo.deg ? a.f_ss = a.f_ll : a.f_ss = get_ss(a.mu_list, f_ps, a.ensinfo.deg)
        a.f_lh = get_lh(a.mu_list, f_ps,a.ensinfo.deg)
        a.ensinfo.deg ? a.f_sh = a.f_lh : a.f_sh = get_sh(a.mu_list, f_ps, a.ensinfo.deg)
        a.f_hh = get_hh(a.mu_list, f_ps,a.ensinfo.deg)

        a.m_ll_vec = get_ll(a.mu_list, m_vec,a.ensinfo.deg)
        a.ensinfo.deg ? a.m_ls_vec = a.m_ll_vec : a.m_ls_vec = get_ls(a.mu_list, m_vec, a.ensinfo.deg)
        a.ensinfo.deg ? a.m_ss_vec = a.m_ll_vec : a.m_ss_vec = get_ss(a.mu_list, m_vec, a.ensinfo.deg)
        a.m_lh_vec = get_lh(a.mu_list, m_vec, a.ensinfo.deg)
        a.ensinfo.deg ? a.m_sh_vec = a.m_lh_vec : a.m_sh_vec = get_sh(a.mu_list, m_vec, a.ensinfo.deg)
        a.m_hh_vec = get_hh(a.mu_list, m_vec, a.ensinfo.deg)
        a.f_ll_vec = get_ll(a.mu_list, f_vec, a.ensinfo.deg)
        a.ensinfo.deg ? a.f_ls_vec = a.f_ll_vec : a.f_ls_vec = get_ls(a.mu_list, f_vec, a.ensinfo.deg)
        a.ensinfo.deg ? a.f_ss_vec = a.f_ll_vec : a.f_ss_vec = get_ss(a.mu_list, f_vec, a.ensinfo.deg)
        a.f_lh_vec = get_lh(a.mu_list, f_vec,a.ensinfo.deg)
        a.ensinfo.deg ? a.f_sh_vec = a.f_lh_vec : a.f_sh_vec = get_sh(a.mu_list, f_vec, a.ensinfo.deg)
        a.f_hh_vec = get_hh(a.mu_list, f_vec, a.ensinfo.deg)
        
        a.m_lh_match = nothing
        a.m_sh_match = nothing
        a.m_hh_match = nothing
        a.f_lh_match = nothing
        a.f_sh_match = nothing
        a.f_hh_match = nothing
        
        a.m_lh_vec_match = nothing
        a.m_sh_vec_match = nothing
        a.m_hh_vec_match = nothing
        a.f_lh_vec_match = nothing
        a.f_sh_vec_match = nothing
        a.f_hh_vec_match = nothing
        
        a.muh_target = nothing
        a.a = nothing
        a.t0 = nothing
        a.deltam = nothing
        return a 
    end
    function EnsObs(ens::EnsInfo, mu::Vector{Vector{Float64}}, m_ps::Vector{uwreal}, m_vec::Vector{uwreal})
        a = new()
        a.ensinfo = ens
        a.mu_list  = mu
        a.is_pseudo = true

        a.m_ll = get_ll(a.mu_list, m_ps,a.ensinfo.deg)
        a.ensinfo.deg ? a.m_ls = a.m_ll : a.m_ls = get_ls(a.mu_list, m_ps, a.ensinfo.deg)
        a.ensinfo.deg ? a.m_ss = a.m_ll : a.m_ss = get_ss(a.mu_list, m_ps, a.ensinfo.deg)
        a.m_lh = get_lh(a.mu_list, m_ps,a.ensinfo.deg)
        a.ensinfo.deg ? a.m_sh = a.m_lh : a.m_sh = get_sh(a.mu_list, m_ps, a.ensinfo.deg)
        a.m_hh = get_hh(a.mu_list, m_ps,a.ensinfo.deg)
        
        a.m_ll_vec = get_ll(a.mu_list, m_vec,a.ensinfo.deg)
        a.ensinfo.deg ? a.m_ls_vec = a.m_ll_vec : a.m_ls_vec = get_ls(a.mu_list, m_vec, a.ensinfo.deg)
        a.ensinfo.deg ? a.m_ss_vec = a.m_ll_vec : a.m_ss_vec = get_ss(a.mu_list, m_vec, a.ensinfo.deg)
        a.m_lh_vec = get_lh(a.mu_list, m_vec, a.ensinfo.deg)
        a.ensinfo.deg ? a.m_sh_vec = a.m_lh_vec : a.m_sh_vec = get_sh(a.mu_list, m_vec, a.ensinfo.deg)
        a.m_hh_vec = get_hh(a.mu_list, m_vec, a.ensinfo.deg)
        
        a.m_lh_match = nothing
        a.m_sh_match = nothing
        a.m_hh_match = nothing
        
        a.m_lh_vec_match = nothing
        a.m_sh_vec_match = nothing
        a.m_hh_vec_match = nothing
        
        a.muh_target = nothing
        a.a = nothing
        a.t0 = nothing
        return a 
    end
end
mutable struct MatInfo
    ensinfo::EnsInfo
    mu::Vector{Float64}
    mat_list::Vector{Matrix{uwreal}}
    y0::Int64
    function MatInfo(_ensinfo::EnsInfo, _mat_list::Array{Array{T,2} where T,1}, _mu::Vector{Float64}, _y0::Int64)
        a = new()
        a.ensinfo = _ensinfo
        a.mu = _mu
        a.mat_list = _mat_list
        a.y0 = _y0
        return a 
    end
end

mutable struct Cat
    obs::Vector{uwreal}
    phih::Vector{uwreal}
    phih_ph::uwreal
    #models::Vector{Vector{Function}}
    x_tofit::Array{uwreal}
    info::String
    fit_param::Vector{Vector{uwreal}} #store all fit parameters
    fit_res::Vector{uwreal} # store all fit results
    chi2_corr::Vector{Float64} # store all chi2 corrected
    n_param::Vector{Int64} # store the number of parameters. Maybe this is redundant
    aic::Vector{Float64} # store the AIC value 
    function Cat(_obs::Vector{uwreal}, _xtofit::Array{uwreal}, _phi_ph::uwreal, _info::String)
        a = new()
        a.obs = _obs
        #a.models = _models
        a.info = _info
        a.x_tofit = _xtofit
        a.phih_ph = _phi_ph
        a.fit_param = Vector{Vector{uwreal}}(undef, 0)
        a.fit_res = Vector{uwreal}(undef, 0)
        a.chi2_corr = Vector{Float64}(undef, 0)
        a.n_param = Vector{Int64}(undef, 0)
        a.aic = Vector{Float64}(undef, 0)
        return a 
    end
end