#========= ENSEMBLE DATABASE ========#
ens_db = Dict(
    #"ens_id"=>[L, beta, is_deg?, m_pi, dtr]
    "H102r002" => [32, 3.4, false, 0.15306, 2],
    #"H102r001" => [32, 3.4, false, 0.15306, 2],
    "H105"     => [32, 3.4, false, 0.12151, 2],
    "H101"     => [32, 3.4, true, 0.17979, 2],
    "H400"     => [32, 3.46, true, 0.16345, 1],
    "N200"     => [48, 3.55, false, 0.09222, 1],
    "N202"     => [48, 3.55, true, 0.13407, 2],
    "N203"     => [48, 3.55, false, 0.11224, 1],
    "D200"     => [64, 3.55, false, 0.06611, 2],
    "N300"     => [48, 3.70, true, 0.10630, 1],
    "J303"     => [64, 3.70, false, 0.06514, 2]
)
trunc_db = Dict(
    "H102r002" => nothing,
    #"H102r001" => nothing,
    "H105"     => nothing,
    "H101"     => nothing,
    "H400"     => nothing,
    "N200"     => nothing,
    "N202"     => nothing,
    "N203"     => nothing,
    "D200"     => 1000,
    "N300"     => nothing,
    "J303"     => nothing

)
#PDG
const hc = 197.3269804 #MeV fm
const M_values = [1869.65, 2010.26, 1968.34, 2112.2, 2980.3, 3096.916] #MD, MD*, MDs, MDs*, \eta_c, J/\psi (MeV)
const M_error = [0.05, 0.05, 0.07, 0.4, 1.2, 0.011]
#1802.05243 
const b_values = [3.40, 3.46, 3.55, 3.70, 3.85]
const b_values2 = [3.40, 3.46, 3.55, 3.70]
const ZM_data = [1.9684, 1.9935, 2.0253, 2.0630, 2.0814]
const ZM_error = [35, 27, 33, 38, 45] .* 1e-4
const ZM_tm_data = [2.6047, 2.6181, 2.6312, 2.6339, 2.6127]
const ZM_tm_error = [42, 33, 42, 48, 55] .* 1e-4
#1608.08900
const t0_data = [2.86, 3.659, 5.164, 8.595]
const t0_error = [11, 16, 18, 29] .* 1e-3
const t0_ph_value = #=[0.415] =# [0.4137]
const t0_ph_error = ones(1,1) .* #=4e-3 =# 3.6e-3
# 1808.09236
const ZA_data = [0.75642, 0.76169, 0.76979, 0.78378, 0.79667]
const ZA_err = [72, 93, 43, 47, 47] .*1e-5
#Covariance matrices (Uncorrelated)
const C1 = zeros(5, 5)
const C2 = zeros(5, 5)
const C3 = zeros(4, 4)
const C4 = zeros(6,6)
const C5 = zeros(5, 5)
for i = 1:6
    C4[i,i] = M_error[i] ^ 2
    if i<= 5
        C1[i, i] = ZM_error[i] ^ 2
        C2[i, i] = ZM_tm_error[i] ^ 2
        C5[i, i] = ZA_err[i]^2
        if i<=4
            C3[i,i] = t0_error[i] ^ 2
        end
    end
end

#Definitions uwreal variables
const ZM = cobs(ZM_data, C1, "ZM values")
const ZM_tm = cobs(ZM_tm_data, C2, "ZM_tm values")
const M = cobs(M_values, C4, "charmed meson masses")
const mpi_ph = uwreal([134.8, 0.3], "mpi phys")
const t0_ph = uwreal([t0_ph_value[1], t0_ph_error[1]], "sqrt(8 t0) (fm)")
const t0_ = cobs(t0_data, C3, "t0")
const a_ = fill(t0_ph, length(t0_)) ./ sqrt.(8 .* t0_)
const Za = cobs(ZA_data, C5, "ZA")

#1802.05243 
C = [[0.375369e-6, 0.429197e-6, -0.186896e-5] [0.429197e-6, 0.268393e-4, 0.686776e-4] [-0.186896e-5, 0.686776e-4, 0.212386e-3]]
z = cobs([0.348629, 0.020921, 0.070613], C, "z")
ZP(b::Float64) = z[1] + z[2] * (b - 3.79) + z[3] * (b - 3.79)^2
Mrat = uwreal([0.9148, 0.0088], "M/mhad")
#Mrat = 0.9148

# Renormalization constants, t0 and a as functions (f(beta))
zm(beta::Float64) = ZM[b_values .== beta][1]
zm_tm(beta::Float64) = Mrat / ZP(beta)
t0(beta::Float64) = t0_[b_values2 .== beta][1]
a(beta::Float64) = a_[b_values2 .== beta][1]
za(beta::Float64) = Za[b_values .== beta][1]