###################################
##########      MASSS    ##########
###################################

##FUNCTIONS
basemodel(x,p) = p[1] .+ p[2] .* x[:,2] .+ p[3] .* x[:,3]  .+ p[4] .* x[:,1]
a2phi2(x)      = x[:,1] .* x[:,2]                  # phi2*a^2/8t0
a2phih2(x)     = x[:,1] .* x[:,3].^2               # phih^2*a^2/8t0          
a4phih4(x)     = x[:,1].^(2) .* x[:,3].^4          # phih^4*(a^2/8t0)^2 
a4phih4_2(x)   = x[:,1].^(2) .* x[:,3].^2          # (a^2/8t0)^2*phih^2 
a4cutoff(x)    = x[:,1].^(2)                       # (a^2/8t0)^2
a2loga(x)      = x[:,1] .* log.(x[:,1])     # (a^2/8t0)^2 * log(a^2/8t0)
phih2loga(x)   = x[:,3].^2 .* log.(x[:,1])  # phih^2*a^2/8t0 
# full list 
func_list = [a2phi2, a2phih2, a4phih4, a4phih4_2, a4cutoff]
label_list = ["a2l", "a2h2", "a4h4", "a4h2", "a4"]
func_map = [Bool.([i,j,k,m,n]) for i=0:1 for j=0:1 for k=0:1 for m=0:1 for n=0:1]

#  a^2log(a) and phih^2log(a)
# func_list = [ a2phih2, a2loga, phih2loga]
# label_list = [ "a2h2", "a2loga", "h2loga"]
# func_map = [Bool.([i,j,k]) for i=0:1 for j=0:1 for k=0:1  ]

# no a2l 
# func_list = [ a2phih2, a4phih4, a4phih4_2, a4cutoff]
# label_list = [ "a2h2", "a4h4", "a4h2", "a4"]
# func_map = [Bool.([i,j,k,m]) for i=0:1 for j=0:1 for k=0:1 for m=0:1]

##COMBINATIONS (LINEAR)
npar = length(func_list) # number of extra parameters 
n_param = Vector(4:4)
label_cutoff = Vector{Vector{String}}(undef, 0)
push!(label_cutoff, ["a2"])

#f_lin definition
f_lin = Vector{Vector{Function}}(undef, npar+1) #f[i][j]-> i: number of extra parameters, j: combinations
f_lin[1] = Vector{Function}(undef, 1)
f_lin[1][1] = (x,p) -> basemodel(x,p)

#f_aux definition -> similar to f_lin but without vectorized opeartors -> useful for plots
f_aux = Vector{Vector{Function}}(undef, npar+1) 
f_aux[1] = Vector{Function}(undef, 1)
f_aux[1][1] = (x,p) -> p[1] + p[2] * x[:,2] + p[3] * x[:,3] + p[4] * x[:,1]

#loop over combinations
for n = 2:npar+1
    aux = filter(x->sum(x) == n-1, func_map)
    f_lin[n] = Vector{Function}(undef, length(aux))
    f_aux[n] = Vector{Function}(undef, length(aux))
    k = 1
    for a in aux 
        #vectorized function 
        push!(n_param, 4+n-1)
        push!(label_cutoff, label_list[a])
        f_lin[n][k] = (x,p) -> basemodel(x,p) .+ sum([p[i+4] for i=1:(n-1)] .* (fill(x,n-1) .|> func_list[a]))
        f_aux[n][k] = (x,p) -> f_aux[1][1](x, p) + sum([p[i+4] for i=1:(n-1)] .* (fill(x,n-1) .|> func_list[a]))
        k = k + 1
    end
end

##COMBINATIONS (NONLINEAR)
f_non_lin = Vector{Vector{Function}}(undef, npar) #f[i][j]-> i: number of extra parameters, j: combinations
f_non_lin_aux = Vector{Vector{Function}}(undef, npar)
n_non_lin_param = Vector(5:npar+4) #param for each non-linear functions
for n = 1:npar
    aux = filter(x->sum(x) == n, func_map)
    f_non_lin[n] = Vector{Function}(undef, length(aux))
    f_non_lin_aux[n] = Vector{Function}(undef, length(aux))
    k = 1
    for a in aux 
        #vectorized function 
        push!(n_param, 4+n)
        f_non_lin[n][k] = (x,p) -> basemodel(x,p) .* (1 .+ sum([p[i+4] for i=1:(n)] .* (fill(x,n) .|> func_list[a])))
        f_non_lin_aux[n][k] = (x,p) -> p[1] + p[2] * x[:,2] + p[3] * x[:,3] + p[4] * x[:,1] +
        (p[1] + p[2] * x[:,2] + p[3] * x[:,3] + p[4] * x[:,1]) .* sum([p[i+4] for i=1:(n)] .* (fill(x,n) .|> func_list[a]))
        k = k + 1
    end
end

##APPEND LIN + NONLIN
f = f_lin
append!(f, f_non_lin)
append!(f_aux, f_non_lin_aux)


#n_param2 = vcat(n_param, n_non_lin_param)

########################################
##########     DEC. CONST.    ##########
########################################
phi2_sym = 0.744666

##FUNCTIONS
f_a2phi2(x)    = x[:,1] .* x[:,2]           # phi2*a^2/8t0
f_a2phih2(x)   = x[:,1] .* (x[:,3]).^2      # phih^2*a^2/8t0          
f_a4phih4(x)   = x[:,1].^(2) .* x[:,3].^4   # phih^4*(a^2/8t0)^2          
f_a4phih2(x)   = x[:,1].^(2) .* x[:,3].^2   # phih^2*(a^2/8t0)^2  
f_a4cutoff(x)  = x[:,1].^(2)                # (a^2/8t0)^2
f_a2loga(x)    = x[:,1] .* log.(x[:,1])     # (a^2/8t0)^2 * log(a^2/8t0)
f_phih2loga(x) = x[:,3].^2 .* log.(x[:,1])  # phih^2*a^2/8t0 

##COMBINATIONS (LINEAR)
func_map = [Bool.([i,j,k, m, n]) for i=0:1 for j=0:1 for k=0:1 for m=0:1 for n=0:1]
f_basemodel1(x,p) = p[1] .+ p[2] .* x[:,2] .+ p[3] .* (1.0 ./ sqrt.(x[:,3])) .+ p[4] .* x[:,1]
f_basemodel2(x,p) = p[1] .+ p[2] .* (3 .* phi2_sym - 2 .* x[:,2]) .+ p[3] .* (1.0 ./ sqrt.(x[:,3])) .+ p[5] .* x[:,1]
#CHIRAL LOGS
f_basemodel1_clog(x,p) = p[1] .+ p[2] .* x[:,2] .+ p[3] .* (1.0 ./ sqrt.(x[:,3])) .+ p[5] .* x[:,1] .+ p[4] .* (3 .* x[:,2] .*log.(x[:,2]) .+
 (3 .*phi2_sym .- x[:,2]) .* log.(1.5 .*phi2_sym .- 0.5*x[:,2]) .+ (2 .* phi2_sym .- x[:,2]) ./ 3 .* log.(2 .*phi2_sym .- x[:,2]))
f_basemodel2_clog(x,p) = p[1] .+ p[2] .* (3 .* phi2_sym - 2 .* x[:,2]) .+ p[3] .* (1.0 ./ sqrt.(x[:,3])) .+ p[6] .* x[:,1] .+ 4 .* p[4] .*(
(1.5 .* phi2_sym .- 0.5.*x[:, 2]) .* log.(1.5 .* phi2_sym .- 0.5.*x[:, 2]) .+ (2 .* phi2_sym .- x[:,2]) ./ 3 .* log.((2 .* phi2_sym .- x[:,2])))

f_func_list = [f_a2phi2, f_a2phih2, f_a4phih4, f_a4phih2, f_a4cutoff]
#f_func_list = [f_a2loga, f_a2phih2, f_a4phih4, f_a4phih2, f_a4cutoff]

f_npar = length(f_func_list) # number of extra parameters 

## FLIN + FAUX
f_f_lin1 = Vector{Vector{Function}}(undef, f_npar+1) #f[i][j]-> i: number of extra parameters, j: combinations
f_f_lin2 = Vector{Vector{Function}}(undef, f_npar+1) #f[i][j]-> i: number of extra parameters, j: combinations
f_f_aux1 = Vector{Vector{Function}}(undef, f_npar+1)
f_f_aux2 = Vector{Vector{Function}}(undef, f_npar+1)

f_f_lin1[1] = Vector{Function}(undef, 1)
f_f_lin2[1] = Vector{Function}(undef, 1)
f_f_aux1[1] = Vector{Function}(undef, 1)
f_f_aux2[1] = Vector{Function}(undef, 1)

f_f_lin1[1][1] = (x,p) -> f_basemodel1(x, p)
f_f_lin2[1][1] = (x,p) -> f_basemodel2(x, p)

f_f_aux1[1][1] = (x,p) -> p[1] + p[2] * x[:,2] + p[3] * (1.0 ./ sqrt.(x[:,3])) + p[4] * x[:,1]
f_f_aux2[1][1] = (x,p) -> p[1] + p[2] * (3*phi2_sym - 2*x[:,2])  + p[3] * (1.0 ./ sqrt.(x[:,3])) + p[4] * x[:,1]

# FLIN + FAUX (CLOGS)  [hep-ph/9206230]
f_f_lin1_clog = Vector{Vector{Function}}(undef, f_npar+1) #f[i][j]-> i: number of extra parameters, j: combinations
f_f_lin2_clog = Vector{Vector{Function}}(undef, f_npar+1) #f[i][j]-> i: number of extra parameters, j: combinations
f_f_aux1_clog = Vector{Vector{Function}}(undef, f_npar+1)
f_f_aux2_clog = Vector{Vector{Function}}(undef, f_npar+1)

f_f_lin1_clog[1] = Vector{Function}(undef, 1)
f_f_lin2_clog[1] = Vector{Function}(undef, 1)
f_f_aux1_clog[1] = Vector{Function}(undef, 1)
f_f_aux2_clog[1] = Vector{Function}(undef, 1)

f_f_lin1_clog[1][1] = (x,p) -> f_basemodel1_clog(x, p)
f_f_lin2_clog[1][1] = (x,p) -> f_basemodel2_clog(x, p)

f_f_aux1_clog[1][1] = (x,p) -> p[1] + p[2] * x[:,2] + p[3] * (1.0 ./ sqrt.(x[:,3])) + p[5] * x[:,1] + p[4] * (3 * x[:,2] .*log.(x[:,2]) +
 (3 *phi2_sym - x[:,2]) .* log.(1.5 *phi2_sym - 0.5*x[:,2]) + (2 * phi2_sym - x[:,2]) / 3 .* log.(2 *phi2_sym - x[:,2]))
f_f_aux2_clog[1][1] = (x,p) -> p[1] + p[2] * (3 * phi2_sym - 2 * x[:,2]) + p[3] * (1.0 ./ sqrt.(x[:,3])) + p[6] * x[:,1] + 4 * p[4] *(
(1.5 * phi2_sym - 0.5*x[:, 2]) .* log.(1.5 * phi2_sym - 0.5*x[:, 2]) + (2 * phi2_sym - x[:,2]) / 3 .* log.((2 * phi2_sym - x[:,2])))

## loop over combinations
for n = 2:f_npar+1
    aux = filter(x->sum(x) == n-1, func_map)
    f_f_lin1[n] = Vector{Function}(undef, length(aux))
    f_f_lin2[n] = Vector{Function}(undef, length(aux))
    f_f_aux1[n] = Vector{Function}(undef, length(aux))
    f_f_aux2[n] = Vector{Function}(undef, length(aux))
    f_f_lin1_clog[n] = Vector{Function}(undef, length(aux))
    f_f_lin2_clog[n] = Vector{Function}(undef, length(aux))
    f_f_aux1_clog[n] = Vector{Function}(undef, length(aux))
    f_f_aux2_clog[n] = Vector{Function}(undef, length(aux))
    k=1
    for a in aux 
        #vectorized function 
        f_f_lin1[n][k] = (x,p) -> f_basemodel1(x,p) .+ sum([p[i+5] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        f_f_aux1[n][k] = (x,p) -> f_f_aux1[1][1](x, p) +
        sum([p[i+5] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        f_f_lin2[n][k] = (x,p) -> f_basemodel2(x,p) .+ sum([p[i+6] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        f_f_aux2[n][k] = (x,p) -> f_f_aux2[1][1](x, p) +
        sum([p[i+6] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        
        f_f_lin1_clog[n][k] = (x,p) -> f_basemodel1_clog(x,p) .+ sum([p[i+6] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        f_f_aux1_clog[n][k] = (x,p) -> f_f_aux1_clog[1][1](x,p) +
        sum([p[i+6] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        f_f_lin2_clog[n][k] = (x,p) -> f_basemodel2_clog(x,p) .+ sum([p[i+7] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        f_f_aux2_clog[n][k] = (x,p) -> f_f_aux2_clog[1][1](x,p) +
        sum([p[i+7] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))

        k = k+1
    end
end
f_npar_comb = 2 * f_npar
f_n_param_comb = collect(5:2:f_npar_comb+5)
f_n_param_comb_clog = collect(6:2:f_npar_comb+6)