# WORKING PLOTS

# plot category average
function plot_cat(cat::Cat, ylab::LaTeXString ; cut::Float64=2.5, save::Bool=false)
    aic = getfield(cat, :aic)
    best = aic_model_ave(cat) * hc /t0_ph
    syst = aic_syst(cat) * hc /t0_ph
    weigths, _ =  aic_weight(cat)
    all_res_aux = getfield(cat, :fit_res) .* hc 
    all_res = [all_res_aux[i] /t0_ph for i =1:length(all_res_aux)]
    
    uwerr.(all_res)
    uwerr(best)
    uwerr(syst)
    ratio_w =  weigths #aic ./ minimum(aic)

    fig = figure(figsize=(10,5))
        #cm = get_cmap("YlOrRd")
        cm = get_cmap("autumn")
        fill_between(collect(1:length(all_res)), value(best-syst), value(best+syst), alpha=0.5, color="royalblue", label="Systematics")
        sc = scatter(collect(1:length(all_res)), value.(all_res), edgecolors="black",c= ratio_w, cmap=cm, vmin=minimum(ratio_w), vmax=maximum(ratio_w) , zorder=3)
        #sc = scatter(collect(1:length(all_res)), value.(all_res), edgecolors="black" , zorder=3)

        errorbar(collect(1:length(all_res)), value.(all_res), yerr=err.(all_res), fmt="none",zorder=0, capsize=2,marker="s", lw=0.8,  mec="black", ecolor="black")
        errorbar(-2, value(best), yerr=err(best), fmt="s",mfc="navy",ms=8, mec="navy",ecolor="navy", capsize=2, zorder=0, label="Model average")
        xlabel("Models", size=20)
        ylabel(ylab, size=20)
        cb = colorbar(sc)
        #cb[:set_label](string("AIC", L"$(M_i)/$", "min(AIC)"))
        cb[:set_label]( "AIC weights")
        legend()
        #ylim(0.46,0.58)
        display(fig)
    if save
        t = string("cat_ave_", cat.info, ".pdf")
        savefig(joinpath(path_plot,t))
    end
end
function plot_cat_tot(cat::Vector{Cat}, ylab::LaTeXString ; cut::Float64=2.5, save::Bool=false)
    best, syst = category_av_tot(cat)
    println(best)
    best *=hc/t0_ph
    best -= 2.
    syst *=hc/t0_ph
    println(syst)
    weigths =  aic_weight.(cat)
    weigths = vcat(weigths[1][1], weigths[2][1])
    all_res_aux = vcat(getfield.(cat, :fit_res) .* hc...)
    all_res = [all_res_aux[i] /t0_ph for i =1:length(all_res_aux)]
    all_res .-=2.
    uwerr.(all_res)
    uwerr(best)
    uwerr(syst)
    ratio_w =  weigths #aic ./ minimum(aic)

    fig = figure(figsize=(10,5))
        cm = get_cmap("YlGn")
        #cm = get_cmap("autumn")
        fill_between(collect(1:length(all_res)), value(best-syst), value(best+syst), alpha=0.5, color="royalblue", label="Systematics")
        sc = scatter(collect(1:length(all_res)), value.(all_res), edgecolors="black",c= ratio_w, cmap=cm, vmin=minimum(ratio_w), vmax=maximum(ratio_w) , zorder=3)
        #sc = scatter(collect(1:length(all_res)), value.(all_res), edgecolors="black" , zorder=3)

        errorbar(collect(1:length(all_res)), value.(all_res), yerr=err.(all_res), fmt="none",zorder=0, capsize=2,marker="s", lw=0.8,  mec="black", ecolor="black")
        errorbar(-2, value(best), yerr=err(best), fmt="s",mfc="navy",ms=8, mec="navy",ecolor="navy", capsize=2, zorder=0, label="Model average")
        xlabel("Models", size=20)
        ylabel(ylab, size=20)
        axvline(64, ls="--", color="red")
        cb = colorbar(sc)
        #cb[:set_label](string("AIC", L"$(M_i)/$", "min(AIC)"))
        cb[:set_label]( "IC weights")
        legend()
        #ylim(0.46,0.58)
        display(fig)
    if save
        t = string("cat_ave_tot", ".pdf")
        savefig(joinpath(path_plot,t))
    end
end
# plot continuum limit for a given category
function plot_cl_charm(c::Cat, ff::Vector{Function}, ylab::LaTeXString, phi2_ph::uwreal, ensinfo::Vector{EnsInfo}; mrat::uwreal=uwreal(1.), dec::Bool=false, p::Union{String, Nothing}=nothing,
    wpm::Union{Dict{String,Vector{Float64}}, Nothing}=nothing, n::Union{Nothing, Int64}=nothing)
    
    #Format
    a2_max = 0.05
    color = ["orange", "darkgreen", "magenta", "navy"]
    fmt = ["^", "v", "<", ">"]
    a_sp = [0.087, 0.077, 0.065, 0.05]

    #Choose model 
    if isnothing(n) #n is nothing -> model with lowest AIC
        aic, n = findmin(getfield(c, :aic))
        println(n)
    else
        aic = 0.0
    end
    #Model info
    phih_2_use = value(c.phih_ph)
    par_n = getfield(c, :n_param)[n]
    upar  = getfield(c, :fit_param)[n]
    if !dec
        res =  value(mrat) * (upar[1] + upar[2]*phi2_ph + upar[3]*phih_2_use)
    else
        res =  getfield(c, :fit_res)[n]
    end
    func  = ff[n]
    println("Results corresponding to the best fit: ", res)
    println("AIC corresponding to the best fit  ", aic)
    println("chi2/chiexp corresponding to the best fit ", c.chi2_vs_exp[n] )
    ##PLOTS
    #xx, yy -> Continuum limit band
    xx = [Float64.(range(1e-8, a2_max, length=100)) fill(phi2_ph, 100) fill(phih_2_use,100)]
    #xx = [Float64.(range(0.0, a2_max, length=100)) fill(phi2[1], 100) fill(phih_2_use,100)] 
    yy = func(xx, upar) * value(mrat)
    #Project one value of muc_i (i=1,2,3)
    n_unique = unique(i-> c.x_tofit[i,2], 1:length(c.x_tofit[:,2]))
    p2 = c.x_tofit[n_unique, 2]
    a2 = c.x_tofit[n_unique, 1]
    pc = c.x_tofit[n_unique, 3]
    x = [a2 p2 pc]
    x2 = [a2 fill(phi2_ph, length(a2)) fill(phih_2_use, length(p2))] #physical value phih
    #x2 = [a2 fill(phi2[1], length(a2)) fill(phih_2_use, length(p2))] #project to symmetric line only

    y = ( c.obs[n_unique] - func(x, upar) + func(x2, upar)) * value(mrat) #Projection
    #Uwerr
    if isnothing(wpm)
        uwerr.(yy)
        uwerr(res)
        uwerr.(y)
    else
        [uwerr(yy_aux, wpm) for yy_aux in yy]
        uwerr(res, wpm)
        [uwerr(y_aux, wpm) for y_aux in y]
    end

    #Plotting
    figure(figsize=(10,6))
    fill_between(xx[:,1], value.(yy)+ err.(yy), value.(yy)-err.(yy), alpha=0.3, color="royalblue") #CL extrapolation (band)
    errorbar(0.0, value(res), err(res), fmt="s", color="red", label="CL", capsize=5, ms=12)#CL extrapolation
    for (k,b) in enumerate(sort(unique(getfield.(ensinfo, :beta))))
        n_ = findall(x->x.beta == b, ensinfo)
        a2_aux = mean(value.(a2[n_]))
        errorbar(value.(x[n_,1]), value.(y[n_]), err.(y[n_]), fmt=fmt[k], label=string(L"$a \approx$", a_sp[k], " fm"), color=color[k], capsize=5, ms=12, mfc="none")#projected datapoints
    end
    ylabel(ylab, size=20)
    xlabel(L"$a^2/8t_0$", size=20)
    legend(ncol=2, fontsize=16)
    grid(ls="dashed")
    tight_layout()
    #ylim(0.46, 0.58)
    if !isnothing(p)
        t = "fit_cl_" * c.info * ".pdf"
        savefig(joinpath(p, t))
        close()
    else
        display(gcf())
        close("all")
    end

end
# plot continuum limit for all categories
function plot_all_cl_charm(cvec::Vector{Cat}, ff::Vector{Function}, ylab::LaTeXString, phi2_ph::uwreal, ensinfo::Vector{EnsInfo};  mrat::uwreal=uwreal(1.), dec::Bool=false, p::Union{String, Nothing}=nothing,wpm::Union{Dict{String,Vector{Float64}}, Nothing}=nothing, n::Union{Nothing, Vector{Int64}}=nothing)
    #Format
    a2_max = 0.05
    color = ["orange", "navy", "darkgreen", "magenta", "navy"]
    fmt = ["^", "v", "<", ">"]
    a_sp = [0.087, 0.077, 0.065, 0.05]
    labels = [L"$\phi_H = \sqrt{8t_0}m_{\eta_c}$", L"$\phi_H = \sqrt{8t_0}m_{\bar D}$", L"$\phi_H = \sqrt{8t_0}m_{\mathrm{sp-fl}}$"  ] 

    figure(figsize=(10,6)) 
    for (idx,c) in enumerate(cvec)
         #Choose model 
        if n[idx] == 0 #n is nothing -> model with lowest AIC
            aic, n_best = findmin(getfield(c, :aic))
        else
            n_best = n[idx]
            aic = 0.0
        end
        phih_2_use = value(c.phih_ph)
        par_n = getfield(c, :n_param)[n_best]
        upar  = getfield(c, :fit_param)[n_best]
        if !dec
            res =  value(mrat) * (upar[1] + upar[2]*phi2_ph + upar[3]*phih_2_use)
        else
            @info("Here for dec const the best res is not consistent wiht chiral logs")
            res =  upar[1] + upar[2]*phi2_ph + upar[3] / sqrt(phih_2_use)
        end        
        func  = ff[n_best]
        println("Category: ", c.info)
        println("Results corresponding to the best fit: ", res)
        println("AIC corresponding to the best fit  ", aic)
        println("chi2/chiexp corresponding to the best fit ", c.chi2_vs_exp[n_best] )
        ##PLOTS
        #xx, yy -> Continuum limit band
        xx = [Float64.(range(1e-8, a2_max, length=100)) fill(phi2_ph, 100) fill(phih_2_use,100)]
        yy = func(xx, upar) * value(mrat)
        #Project one value of muc_i (i=1,2,3)
        n_unique = unique(i-> c.x_tofit[i,2], 1:length(c.x_tofit[:,2]))
        p2 = c.x_tofit[n_unique, 2]
        a2 = c.x_tofit[n_unique, 1]
        pc = c.x_tofit[n_unique, 3]
        x = [a2 p2 pc]
        # before it was 
        # x2 = [a2 p2 fill(phih_2_use, length(p2))] #physical value phih
        # but to me should be 
        x2 = [a2 fill(phi2_ph, length(p2)) fill(phih_2_use, length(p2))] #physical value phih
        y = ( c.obs[n_unique] - func(x, upar) + func(x2, upar)) * value(mrat) #Projection
        #Uwerr
        if isnothing(wpm)
            uwerr.(yy)
            uwerr(res)
            uwerr.(y)
        else
            [uwerr(yy_aux, wpm) for yy_aux in yy]
            uwerr(res, wpm)
            [uwerr(y_aux, wpm) for y_aux in y]
        end

        #Plotting
        fill_between(xx[:,1], value.(yy)+ err.(yy), value.(yy)-err.(yy), alpha=0.2, color=color[idx]) #CL extrapolation (band)
        #errorbar(0.0 +(idx-1)/800, value(res), err(res), fmt="s", color=color[idx], capsize=5, ms=12)#CL extrapolation
        errorbar(value.(x[:,1]), value.(y[:]), err.(y[:]), fmt=fmt[idx], color=color[idx], capsize=5, ms=12, label = labels[idx],  mfc="none")#projected datapoints            
        
    end
    #tight_layout()
    xlim(-0.001, 0.05)
    #ylim(2.3,3.5)
    grid(ls="dashed")
    xlabel(L"$a^2/8t_0$", size=20)
    ylabel(ylab, size=20)
    legend(fontsize=16, loc="upper left")
    if !isnothing(p)
        t = "mc_cl_all_cat.pdf"
        savefig(joinpath(p, t))
        close()
    else
        display(gcf())
        close("all")
    end
end
# plot chiral extrapolation for a given category
function plot_chi_charm(c::Cat, ff::Vector{Function}, ylab::LaTeXString, phi2_ph::uwreal, ensinfo::Vector{EnsInfo}; p::Union{String, Nothing}=nothing,
    mrat::uwreal=uwreal(1.), dec::Bool=false,wpm::Union{Dict{String,Vector{Float64}}, Nothing}=nothing, n::Union{Nothing, Int64}=nothing)
    
    #Format
    phi2_max = maximum(value.(phi2))
    color = ["orange", "darkgreen", "magenta", "navy"]
    fmt = ["^", "v", "<", ">"]
    a_sp = [0.087, 0.077, 0.065, 0.05]

    #Choose model 
    if isnothing(n) #n is nothing -> model with lowest AIC
        aic, n = findmin(getfield(c, :aic))
        println(n)
    else
        aic = 0.0
    end
    #Model info
    @info("error in t0_ph  not included")
    phih_2_use =  c.phih_ph #value(c.phih_ph)
    par_n = getfield(c, :n_param)[n]
    upar  = getfield(c, :fit_param)[n]
    if !dec
        res =  value(mrat) * (upar[1] + upar[2]*phi2_ph + upar[3]*phih_2_use)
    else
        res =  getfield(c, :fit_res)[n]
    end
    func  = ff[n]
    println("Results corresponding to the best fit: ", res)
    println("AIC corresponding to the best fit  ", aic)
    println("chi2/chiexp corresponding to the best fit ", c.chi2_vs_exp[n] )
    ##PLOTS
    #xx, yy -> Continuum limit band
    xx = [fill(1e-8, 100) Float64.(range(0.01, phi2_max, length=100)) fill(phih_2_use,100)]
    yy = func(xx, upar) * value(mrat)

    #Project one value of muc_i (i=1,2,3)
    n_unique = unique(i-> c.x_tofit[i,2], 1:length(c.x_tofit[:,2]))
    p2 = c.x_tofit[n_unique, 2]
    a2 = c.x_tofit[n_unique, 1]
    pc = c.x_tofit[n_unique, 3]
    x = [a2 p2 pc]
    x2 = [a2 p2 fill(phih_2_use, length(p2))] #physical value phih
    y = ( c.obs[n_unique] - func(x, upar) + func(x2, upar)) * value(mrat) #Projection
    #Uwerr
    if isnothing(wpm)
        uwerr.(yy)
        uwerr(res)
        uwerr.(y)
    else
        [uwerr(yy_aux, wpm) for yy_aux in yy]
        uwerr(res, wpm)
        [uwerr(y_aux, wpm) for y_aux in y]
    end

    #Plotting
    figure(figsize=(10,6))
    fill_between(xx[:,2], value.(yy)+ err.(yy), value.(yy)-err.(yy), alpha=0.3, color="royalblue") #CL extrapolation (band)
    errorbar(value(phi2_ph), value(res), err(res), fmt="s", color="red", label=L"$\phi_2 = \phi^\mathrm{phys}_2$", capsize=5, ms=12)#CL extrapolation
    for (k,b) in enumerate(sort(unique(getfield.(ensinfo, :beta))))
        n_ = findall(x->x.beta == b, ensinfo)
        a2_aux = mean(value.(a2[n_]))
        errorbar(value.(x[n_,2]), value.(y[n_]), err.(y[n_]), fmt=fmt[k], label=string(L"$a \approx$", a_sp[k], " fm"), color=color[k], capsize=5, ms=12, mfc="none")#projected datapoints
        #dashed lines
        xxx = [fill(a2_aux, 100) Float64.(range(0.0, phi2_max, length=100)) fill(value(c.phih_ph), 100)]
        yyy = func(xxx, upar) * value(mrat)
        plot(xxx[:,2], value.(yyy), ls="--", color=color[k])#dashed lines 
    end
    # fill between physical values 
    am = M[1] * t0_ph / hc
    uwerr(am)
    fill_between(xx[:,2], value(am)+err(am), value(am)-err(am), color="forestgreen", alpha=0.2, zorder=1, label="PDG")
    ylabel(ylab, size=22)
    xlabel(L"$\phi_2$", size=22)
    legend(ncol=2, fontsize=16)
    ylim(3.85, 4.2)
    tight_layout()
    grid(ls="dashed")
    if !isnothing(p)
        t = "fit_phi2_" * c.info * ".pdf"
        savefig(joinpath(p, t))
        close()
    else
        display(gcf())
        close("all")
    end

end
# plot chi2_vs_exp  adn weights for a given category
function plot_chi2_vs_exp(c::Cat, label::Vector{Vector{String}}; p::Union{String, Nothing}=nothing)
    chi2_exp = getfield(c, :chi2_vs_exp)
    label = vcat(label, label[2:end])
    figure(figsize=(12,8))

    label_plot = []
    for i in 1:length(label)
        aux = ""
        [aux *= label[i][k] * " " for k in 1:length(label[i])]
        push!(label_plot, aux)
    end
    xx = collect(1:length(chi2_exp))
    subplots_adjust(hspace=0.0)
    subplot(311)
    ax1 = gca()
    setp(ax1.get_xticklabels(),visible=false) # Disable x tick labels
    plot(xx, aic_weight(c)[1], marker="^", ls="dashed", color="forestgreen")
    ylabel(L"$w^\mathrm{AIC}$")    

    subplot(312)
    plot(xx, chi2_exp, marker="^", ls="dashed", color="royalblue")
    xticks(xx, label_plot, rotation=90)
    ylabel(L"$\chi^2 / \chi^2_{\mathrm{exp}}$")
    
    if !isnothing(p)
        t = "chi2_vs_exp_" * c.info * ".pdf"
        savefig(joinpath(p, t))
        close()
    else
        display(gcf())
        close("all")
    end 
end


function plot_chi2_vs_exp_clog(c::Cat,label::Vector{Vector{String}}; p::Union{String, Nothing}=nothing)
    chi2_exp = getfield(c, :chi2_vs_exp)
    label_log = deepcopy(label)
    label_log = vcat.("LOG", label_log)
    #label = vcat(label, label_log)
    figure(figsize=(12,8))

    label_plot = []
    for i in 1:length(label)
        aux = ""
        [aux *= label[i][k] * " " for k in 1:length(label[i])]
        push!(label_plot, aux)
    end
    xx = collect(1:length(chi2_exp))
    subplots_adjust(hspace=0.0)
    subplot(311)
    ax1 = gca()
    setp(ax1.get_xticklabels(),visible=false) # Disable x tick labels
    plot(xx, aic_weight(c)[1], marker="^", ls="dashed", color="forestgreen")
    ylabel(L"$w^\mathrm{AIC}$")    

    subplot(312)
    plot(xx, chi2_exp, marker="^", ls="dashed", color="royalblue")
    xticks(xx, label_plot, rotation=90)
    ylabel(L"$\chi^2 / \chi^2_{\mathrm{exp}}$")
    
    if !isnothing(p)
        t = "chi2_vs_exp_" * c.info * ".pdf"
        savefig(joinpath(p, t))
        close()
    else
        display(gcf())
        close("all")
    end 
end

##



# plot histogram for a vector of categories
function plot_hist_tot(c::Vector{Cat}, lbl::LaTeXString; p::Union{Nothing, String}=nothing)
    aux_aic = getfield.(c, :aic)
    aux_fit_res = getfield.(c, :fit_res)

    aic = aux_aic[1]
    fit_res = aux_fit_res[1]
    for i = 2:length(aux_aic)
        aic = vcat(aic, aux_aic[i])
        fit_res = vcat(fit_res, aux_fit_res[i])
    end

    best = aic_model_ave(aic, fit_res) *hc /t0_ph
    sys = aic_syst(aic, fit_res) * hc / t0_ph
    sys = value(sys)
    println(best)
    w, _ = aic_weight(aic) 

    all_res_aux = vcat(getfield.(c, :fit_res) .* hc... )
    all_res = [all_res_aux[i] /t0_ph for i =1:length(all_res_aux)]
    println(length(all_res))

    uwerr.(all_res)
    uwerr(best)
    fig = figure()
    all_res .-=1.9
    hist(value.(all_res), bins=15, histtype="stepfilled",alpha=0.5, ec="k", color="navy", weights=w)
    axvline(value(best) -1.9, 0, 1, color="orange", label="model average")
    ylim = plt.ylim()
    fill_betweenx(ylim, value(best)-sys-1.9,value(best)+sys-1.9, color="orange", alpha=0.4)

    ylabel("Frequency")
    xlabel(lbl)
    legend()
    tight_layout()
    plt.ylim(ylim)
    if !isnothing(p)
        o = split(c[1].info, ", ")[1] 
        t = string("hist_", o, ".pdf")
        savefig(joinpath(p, t))
        close()
    else
        display(gcf())
    end
end
####



##########################
#PLOTTER FOR MESON MASSES
##########################


function plot_chi_masses(c::Cat, ff::Vector{Function}, ylab::LaTeXString, phi2_ph::uwreal, ensinfo::Vector{EnsInfo}; p::Union{String, Nothing}=nothing,
    wpm::Union{Dict{String,Vector{Float64}}, Nothing}=nothing, n::Union{Nothing, Int64}=nothing)
    
    #Format
    phi2_max = 0.744666
    color = ["orange", "darkgreen", "magenta", "navy"]
    fmt = ["^", "v", "<", ">"]
    a_sp = [0.087, 0.077, 0.065, 0.05]
    
    #Choose model 
    if isnothing(n) #n is nothing -> model with lowest AIC
        aic, n = findmin(getfield(c, :aic))
    end
    #Model info
    phih_2_use = c.phih_ph
    par_n = getfield(c, :n_param)[n]
    upar  = getfield(c, :fit_param)[n]
    res   = upar[1] + upar[2]*phi2_ph + upar[3]*phih_2_use
    func  = ff[n]

    ##PLOTS
    #xx, yy -> Continuum limit band
    xx = [fill(0.0, 100) Float64.(range(0.01, phi2_max, length=100)) fill(phih_2_use,100)]
    yy = func(xx, upar) 

    #Project one value of muc_i (i=1,2,3)
    n_unique = unique(i-> c.x_tofit[i,2], 1:length(c.x_tofit[:,2]))
    p2 = c.x_tofit[n_unique, 2]
    a2 = c.x_tofit[n_unique, 1]
    pc = c.x_tofit[n_unique, 3]
    x = [a2 p2 pc]
    x2 = [a2 p2 fill(phih_2_use, length(p2))] #physical value phih
    y =  c.obs[n_unique] - func(x, upar) + func(x2, upar) #Projection
    #Uwerr
    if isnothing(wpm)
        uwerr.(yy)
        uwerr(res)
        uwerr.(y)
    else
        [uwerr(yy_aux, wpm) for yy_aux in yy]
        uwerr(res, wpm)
        [uwerr(y_aux, wpm) for y_aux in y]
    end

    #Plotting
    figure(figsize=(10,6))
    fill_between(xx[:,2], value.(yy)+ err.(yy), value.(yy)-err.(yy), alpha=0.3, color="royalblue") #CL extrapolation (band)
    errorbar(value(phi2_ph), value(res), err(res), fmt="s", color="red", label=L"$\phi_2 = \phi^\mathrm{phys}_2$", capsize=5, ms=12)#CL extrapolation
    for (k,b) in enumerate(sort(unique(getfield.(ensinfo, :beta))))
        n_ = findall(x->x.beta == b, ensinfo)
        a2_aux = mean(value.(a2[n_]))
        errorbar(value.(x[n_,2]), value.(y[n_]), err.(y[n_]), fmt=fmt[k], label=string(L"$a \approx$", a_sp[k], " fm"), color=color[k], capsize=5, ms=12, mfc="none")#projected datapoints
        #dashed lines
        xxx = [fill(a2_aux, 100) Float64.(range(0.0, phi2_max, length=100)) fill(value(c.phih_ph), 100)]
        yyy = func(xxx, value.(upar)) 
        plot(xxx[:,2], yyy, ls="--", color=color[k])#dashed lines 
    end
    ylabel(ylab, size=20)
    xlabel(L"$\phi_2$", size=20)
    #ylim(3.9, 4.2)
    grid(ls="dashed")
    legend(ncol=2, fontsize=16)
    tight_layout()
    if !isnothing(p)
        t = "fit_phi2_" * c.info * ".pdf"
        savefig(joinpath(p, t))
        close()
    else
        display(gcf())
        close("all")
    end
end




function plot_hist(cat::Vector{Cat} ; save::Bool=false)
    best = category_av(cat)[1] *hc /t0_ph[1]
    println(best)
    all_res_aux = vcat(getfield.(cat, :fit_res) .* hc... )
    all_res = [all_res_aux[i] /t0_ph[1] for i =1:length(all_res_aux)]
    println(length(all_res))
    uwerr.(all_res)
    uwerr(best)
    weigths = vcat(aic_weigth.(cat)...)
    println(size(weigths))
    fig = figure()
        hist(value.(all_res), bins=20, density=true, weights=weigths, histtype="stepfilled", facecolor="royalblue",ec="k", alpha=0.3)
        fill_betweenx(collect(0:5), value(best)-err(best)+4,value(best)+err(best)-4, color="tomato", alpha=0.4)
        axvline(value(best),0, 5, color="tomato", label="best estimate" )
        xlim(1460, 1520)
        ylim(0,0.1)
        ylabel("Probability")
        xlabel(L"$M_c^{\rm{RGI}}\ [\rm{MeV}]$")
        legend()
    display(fig)
    if save
        path_plot= "/Users/ale/Desktop/plots"
        t = string("hist_muc.pdf")
        savefig(joinpath(path_plot,t))
    end
    close()
end


####
## PLOT FROM JAVIER TO TEST decays
###

