using BDIO, ADerrors

function read_BDIO(path::String, uinfo::Int64)
    r = Vector{uwreal}(undef, 0)
    fb = BDIO_open(path, "r")
    BDIO_seek!(fb)
    
    if BDIO_get_uinfo(fb) == uinfo
        push!(r, read_uwreal(fb))
    end

    while BDIO_seek!(fb, 2) 
        if BDIO_get_uinfo(fb) == uinfo
            push!(r, read_uwreal(fb))
        end
    end

    BDIO_close!(fb)
    return r
end

function read_BDIO(path::String, type::String, obs::String)
    dict_w = Dict(
        "mll"  => 0,
        "mls"  => 1,
        "t0"   => 2,
        "phi2" => 3,
        "phi4" => 4
    )
    dict_tm = Dict(
        "mll"  => 0,
        "mls"  => 1,
        "mlh"  => 2,
        "msh"  => 3,
        "mhh"  => 4,
        "flh"  => 5,
        "fsh"  => 6,
        "fhh"  => 7,
        "mlh_star" => 8,
        "msh_star" => 9,
        "mhh_star" => 10,
        "flh_star" => 11,
        "fsh_star" => 12,
        "fhh_star" => 13,
        "muh"      => 14
    )
    dict_md = Dict(
        "deltam"       => 0,
        "t0_shifted"   => 1,
        "phi2_shifted" => 2,
        "phi4_shifted" => 3
    )

    dict2dict = Dict("w" => dict_w, "tm" => dict_tm, "md" => dict_md)
    
    if !(type in keys(dict2dict))
        error("Incorrect type.\ntype = $(keys(dict2dict))")
    end
    if !(obs in keys(dict2dict[type]))
        error("Incorrect obs.\nobs = $(keys(dict2dict[type]))")
    end
    return read_BDIO(path, dict2dict[type][obs])
end
