##FUNCTIONS
basemodel(x,p) = p[1] .+ p[2] .* x[:,2] .+ p[3] .* x[:,3]  .+ p[4] .* x[:,1]
a2phi2(x) = x[:,1] .* x[:,2]      # phi2*a^2/8t0
a2phih2(x) = x[:,1] .* x[:,3].^2  # phih^2*a^2/8t0          
a4phih4(x) = x[:,1].^(2) .* x[:,3].^4  #phih^4*(a^2/8t0)^2 
a4phih4_2(x) = x[:,1].^(2) .* x[:,3].^2  #(a^2/8t0)^2*phih^2 
a4cutoff(x) = x[:,1].^(2)   #(a^2/8t0)^2

func_list = [a2phi2, a2phih2, a4phih4, a4phih4_2, a4cutoff]
label_list = ["a2l", "a2h2", "a4h4", "a4h2", "a4"]

##COMBINATIONS (LINEAR)
func_map = [Bool.([i,j,k,m,n]) for i=0:1 for j=0:1 for k=0:1 for m=0:1 for n=0:1]
npar = length(func_list) # number of extra parameters 
n_param_j = Vector(4:4)
label_cutoff = Vector{Vector{String}}(undef, 0)
push!(label_cutoff, ["a2"])
#f_lin definition
f_lin = Vector{Vector{Function}}(undef, npar+1) #f[i][j]-> i: number of extra parameters, j: combinations
f_lin[1] = Vector{Function}(undef, 1)
f_lin[1][1] = (x,p) -> basemodel(x,p)

#f_aux definition -> similar to f_lin but without vectorized opeartors -> useful for plots
f_aux = Vector{Vector{Function}}(undef, npar+1) 
f_aux[1] = Vector{Function}(undef, 1)
f_aux[1][1] = (x,p) -> p[1] + p[2] * x[:,2] + p[3] * x[:,3] + p[4] * x[:,1]

#loop over combinations
for n = 2:npar+1
    aux = filter(x->sum(x) == n-1, func_map)
    f_lin[n] = Vector{Function}(undef, length(aux))
    f_aux[n] = Vector{Function}(undef, length(aux))
    k = 1
    for a in aux 
        #vectorized function 
        push!(n_param_j, 4+n-1)
        push!(label_cutoff, label_list[a])
        f_lin[n][k] = (x,p) -> basemodel(x,p) .+ sum([p[i+4] for i=1:(n-1)] .* (fill(x,n-1) .|> func_list[a]))
        f_aux[n][k] = (x,p) -> f_aux[1][1](x, p) + sum([p[i+4] for i=1:(n-1)] .* (fill(x,n-1) .|> func_list[a]))
        k = k + 1
    end
end


##COMBINATIONS (NONLINEAR)
f_non_lin = Vector{Vector{Function}}(undef, npar) #f[i][j]-> i: number of extra parameters, j: combinations
f_non_lin_aux = Vector{Vector{Function}}(undef, npar)
n_non_lin_param = Vector(5:npar+4) #param for each non-linear functions
for n = 1:npar
    aux = filter(x->sum(x) == n, func_map)
    f_non_lin[n] = Vector{Function}(undef, length(aux))
    f_non_lin_aux[n] = Vector{Function}(undef, length(aux))
    k = 1
    for a in aux 
        #vectorized function 
        push!(n_param_j, 4+n)
        f_non_lin[n][k] = (x,p) -> basemodel(x,p) .* (1 .+ sum([p[i+4] for i=1:(n)] .* (fill(x,n) .|> func_list[a])))
        f_non_lin_aux[n][k] = (x,p) -> p[1] + p[2] * x[:,2] + p[3] * (x[:,3]) + p[4] * x[:,1] +
        (p[1] + p[2] * x[:,2] + p[3] *(x[:,3]) + p[4] * x[:,1]) .* sum([p[i+4] for i=1:(n)] .* (fill(x,n) .|> func_list[a]))
        k = k + 1
    end
end

##APPEND LIN + NONLIN
f = f_lin
model_charm     = append!(f, f_non_lin)
model_charm_aux = append!(f_aux, f_non_lin_aux)







#=
import Base.+  
import Base.*  

+(f::Function, g::Function) = (x...) -> f(x...) + g(x...)  
+(f::Function, g::Function) = (x...) -> f(x...) .+ g(x...)  
*(f::Function, g::Function) = (x...) -> f(x...) * g(x...)  
*(f::Function, g::Function) = (x...) -> f(x...) .* g(x...)  

# charm quark fits 
function basemodel(x,p)
    n=size(x,1)
    aux = [p[1] for i in 1:n] .+ [p[2] for i in 1:n].*x[:,2] .+ [p[3] for i in 1:n] .* x[:,3] .+ [p[4] for i in 1:n].*x[:,1]
    return aux
end
#a^2 cutoff effects
iden(x)  = 1 
a2phi2(x)      = x[:,1] .* x[:,2]        #a^2/8t0*phi2 
a2phih(x)      = x[:,1] .* x[:,3].^2     #a^2/8t0*phih2
#a^4 cutoff effects
a4(x)          = x[:,1].^2               #(a^2/8t0)^2
a4phih2(x)     = x[:,1].^2 .*x[:,3].^2   #(a^2/8t0)^2*phih^2
a4phih4(x)     = x[:,1].^2 .*x[:,3].^4   #(a^2/8t0)^2*phih^4

func_comb_charm   = collect(combinations([ a2phi2, a2phih, a4, a4phih2, a4phih4]))
model_charm = Vector{Function}(undef, 0)
push!(model_charm, basemodel)
n_param_charm = Vector{Int64}(undef,0)
push!(n_param_charm, 4)
for i in 1:length(func_comb_charm)
    tmp_farr = Vector{Function}(undef,0)
    #push!(tmp_farr, basemodel)
    for k in 1:length(func_comb_charm[i])
        tmp_f = (x,p) -> [p[n_param_charm[1]+k] for n in 1:size(x,1)] .* func_comb_charm[i][k](x)
        push!(tmp_farr, tmp_f)
    end
    push!(model_charm,  basemodel + sum(tmp_farr))
    push!(n_param_charm, 4+length(func_comb_charm[i]))
    push!(n_param_charm, 4+length(func_comb_charm[i]))
    fff(x,p) = 1.
    aux_non_lin = (x,p) -> basemodel(x,p) .+ [p[1] for l in 1:length(x[:,1])] .+ [p[2] for l in 1:length(x[:,1])].*x[:,2] .+ [p[3] for l in 1:length(x[:,1])].*x[:,3] .+ [p[4] for l in 1:length(x[:,1])].*x[:,1] #.* [ ([p[n_param_charm[1]+k] for n in 1:size(x,1)] .* func_comb_charm[i][k](x)) for k in 1:length(func_comb_charm[i])] 
    push!(model_charm,  basemodel * (fff + sum(tmp_farr)))
end


# decay constansts 
function basemodel_dec(x,p)
    n=size(x,1)
    aux = [p[1] for i in 1:n] .+ [p[2] for i in 1:n].*x[:,2] .+ [p[3] for i in 1:n] .* x[:,3] .+ [p[4] for i in 1:n].*x[:,1]
    return aux
end

func_comb_decay   = collect(combinations([ a2phi2, a2phih, a4, a4phih2, a4phih4]))
model_decay       = Vector{Function}(undef, 0)
push!(model_decay, basemodel_dec)
n_param_decay     = Vector{Int64}(undef,0)
push!(n_param_decay, 4)
for i in 1:length(func_comb_decay)
    tmp_farr = Vector{Function}(undef,0)
    for k in 1:length(func_comb_decay[i])
        tmp_f = (x,p) -> [p[n_param_decay[1]+k] for n in 1:size(x,1)] .* func_comb_decay[i][k](x)
        push!(tmp_farr, tmp_f)
    end
    push!(model_decay,  basemodel_dec + sum(tmp_farr))
    push!(n_param_decay, 4+length(func_comb_decay[i]))
    #push!(n_param_decay, 4+length(func_comb_decay[i]))
    #fff(x,p) = 1.
    #aux_non_lin = (x,p) -> basemodel_dec(x,p) .+ [p[1] for l in 1:length(x[:,1])] .+ [p[2] for l in 1:length(x[:,1])].*x[:,2] .+ [p[3] for l in 1:length(x[:,1])].*x[:,3] .+ [p[4] for l in 1:length(x[:,1])].*x[:,1] #.* [ ([p[n_param_charm[1]+k] for n in 1:size(x,1)] .* func_comb_charm[i][k](x)) for k in 1:length(func_comb_charm[i])] 
    #push!(model_decay,  basemodel_dec * (fff + sum(tmp_farr)))
end

=#

##
# =#

#=
# OLD MODELS
basemodel(x,p) = p[1] .+ p[2].*x[:,2] .+ p[3].*x[:,3]  .+ p[4].*x[:,1]
#a^2 cutoff effects
a2phi2(x)      = x[:,1] .* x[:,2]        #a^2/8t0*phi2 
a2phih(x)      = x[:,1] .* x[:,3].^2     #a^2/8t0*phih2
#a^4 cutoff effects
a4(x)          = x[:,1].^2               #(a^2/8t0)^2
a4phih2(x)     = x[:,1].^2 .*x[:,3].^2   #(a^2/8t0)^2*phih^2
a4phih4(x)     = x[:,1].^2 .*x[:,3].^4   #(a^2/8t0)^2*phih^4
func_comb_charm   = collect(combinations([ a2phi2, a2phih, a4, a4phih2, a4phih4]))
# charm functions 
model_charm = Vector{Function}(undef, 0)
push!(model_charm, basemodel)
n_param_charm = Vector{Int64}(undef,0)
push!(n_param_charm, 4)
#linear combinations
for i in 1:length(func_comb_charm)
    aux_f = (x,p) -> basemodel(x,p) .+ sum([p[4+k] for k in 1:length(func_comb_charm[i])] .* (fill(x,length(func_comb_charm[i])) .|>func_comb_charm[i]))
    push!(model_charm, aux_f)
    push!(n_param_charm, 4+length(func_comb_charm[i]))
end
#non-linear combinations
for i in 1:length(func_comb_charm)
    aux_f = (x,p) -> basemodel(x,p) .* (1 .+ sum([p[4+k] for k in 1:length(func_comb_charm[i])] .* (fill(x,length(func_comb_charm[i])) .|>func_comb_charm[i])))
    push!(model_charm, aux_f)
    push!(n_param_charm, 4+length(func_comb_charm[i]))
end

# models for decay constants 

basemodel_dec(x,p) = p[1] .+ p[2].*x[:,2] .+ p[3]./ sqrt.(x[:,3])  .+ p[4].*x[:,1]

model_decays = Vector{Function}(undef, 0)
push!(model_decays, basemodel_dec)
n_param_decays = Vector{Int64}(undef,0)
push!(n_param_decays, 4)
#linear combinations
for i in 1:length(func_comb_charm)
    aux_f = (x,p) -> basemodel_dec(x,p) .+ sum([p[4+k] for k in 1:length(func_comb_charm[i])] .* (fill(x,length(func_comb_charm[i])) .|>func_comb_charm[i]))
    push!(model_decays, aux_f)
    push!(n_param_decays, 4+length(func_comb_charm[i]))
end
#non-linear combinations
for i in 1:length(func_comb_charm)
    aux_f = (x,p) -> basemodel_dec(x,p) .* (1 .+ sum([p[4+k] for k in 1:length(func_comb_charm[i])] .* (fill(x,length(func_comb_charm[i])) .|>func_comb_charm[i])))
    push!(model_decays, aux_f)
    push!(n_param_decays, 4+length(func_comb_charm[i]))
end
=#