#################################
# DEPRECATED 
# This file was created by Alessandro Conigli - 2022
# The pourpose of this code is to compute twisted mass 
# observables from raw correlators stored in mesons.dat files.
# The analysis is performed at the level of a single ensemble 
# and results are then stored in BDIO files 
# It extracts:
#  - charm quark mass 
#  - ll hl hh pseudo and vector masses 
#  - ll hl hh pseudo and vector leptonic decays
#  
#  The analysis is performed by solving a GEVP 
##################################
using Revise, ADerrors, BDIO, juobs, PyPlot, LaTeXStrings, DelimitedFiles, Combinatorics
using LsqFit, juobs.LeastSquaresOptim 

include("gevp_types.jl")
include("gevp_reader.jl")
include("gevp_tools.jl")
include("const.jl")
include("gevp_func_comb.jl")


#============= INPUT PAR ===========#

# desktop parameters
#include("/home/alessandro/juobs/constants/juobs_const.jl")
#path_data ="/home/alessandro/Desktop/data/heavy_3pt/3pt"
#path_rw = "/media/alessandro/4277fef2-edc5-4e0d-89cb-f5d1d44fbc8c/data/rwf"
#path_plot = "/home/alessandro/google-drive/phd/secondment/3pf test/analysis/plots"

# macbook parameters
include("/Users/ale/juobs/constants/juobs_const.jl")
path_data          = "/Users/ale/Desktop/data/charm_full_dirac" #julien_comparison"
path_rw            = "/Users/ale/Desktop/data/ms1_dat"
path_plot          = "/Users/ale/Il mio Drive/phd/analysis/charm_gevp_2022/plots"
path_results       = "/Users/ale/Il mio Drive/phd/analysis/charm_gevp_2022/bdio"
path_plat_meff     = "/Users/ale/gevp-automation/src/plat_meff.txt"
path_plat_meff_vec = "/Users/ale/gevp-automation/src/plat_meff_vec.txt"
path_plat_dec      = "/Users/ale/gevp-automation/src/plat_dec.txt"
path_plat_dec_vec  = "/Users/ale/gevp-automation/src/plat_dec_vec.txt"
# latex-style labels 
PyPlot.rc("font", family="sans serif", size=13)
rcParams = PyPlot.PyDict(PyPlot.matplotlib."rcParams")
rcParams["text.usetex"] = false
rcParams["mathtext.fontset"]  = "cm"
#rcParams["font.size"] =11
rcParams["axes.labelsize"] =16

# hyper-parameters
const ens   = "J303"
const rwf   = true
const tau   = 2  # shifting time in the GEVP 
const sector = Dict("masses"=>true, "decays"=>true)


truncate = Dict(
    "H101"     => [878, 999],
    "H102r001" => 305,
    "H102r002" => 430,
    "H400"     => [505,540],
    "N202"     => 899,
    "N200"     => [856,856],
    "N203"     => [756,787],
    "N300"     => 1140,
    "J303"     => 739
)
#========  READING CORRELATORS ==========#
##
ensinfo = EnsInfo(ens, ens_db[ens])
corr_pp        = get_corr(ensinfo, "G5", "G5", rw=rwf)
corr_pa1       = get_corr(ensinfo, "G5", "G1G5", rw=rwf)
corr_a1a1      = get_corr(ensinfo, "G1G5", "G1G5", rw=rwf)

if sector["masses"]
    matinfo_masses = comp_mat_multigamma(ensinfo, corr_pp, corr_pa1, corr_a1a1)
    mu_list        = getfield.(corr_pp, :mu)
    l, s, muh      = get_mu(mu_list, ensinfo.deg)
    en0_ps,en0_vec = gevp_mass(matinfo_masses, path_plat_meff, path_plat_meff_vec, t0=3, pl=true)
end

if sector["decays"]
    matinfo_decay_ps   = comp_mat(ensinfo, corr_pp, tau)
    matinfo_decay_vec  = comp_mat(ensinfo, corr_a1a1, tau)
    dec0_ps, data      = gevp_dec(matinfo_decay_ps, en0_ps, path_plat_dec, t0=3, n=1, pl=true)
    dec0_vec, data_vec =  gevp_dec(matinfo_decay_vec, en0_vec, path_plat_dec_vec, t0=3, n=1, pl=true, pseudo=false)
    uwerr.(dec0_ps)
    uwerr.(dec0_vec)
    close("all")

    
    for i in 1:length(data)
        y0s          = getfield(matinfo_decay_ps[i], :y0)
        aux_data_ps  = data[i] .* [ sqrt(2 / en0_ps[i]^3) * sum(mu_list[i]) for k in 1:length(data[i]) ]
        aux_data_vec = data_vec[i] .* [  sqrt(2 / en0_vec[i])  for k in 1:length(data[i]) ]
        uwerr.(aux_data_ps)
        uwerr.(aux_data_vec)
        xx           = collect(1:length(aux_data_ps))
        v , e        = value(dec0_ps[i]), err(dec0_ps[i])
        v_vec, e_vec = value(dec0_vec[i]), err(dec0_vec[i])
        plat_ps      =  select_plateau(ensinfo, mu_list, path_plat_dec)[i] .- y0s
        plat_vec     =  select_plateau(ensinfo, mu_list, path_plat_dec_vec)[i] .- y0s

        errorbar(xx, value.(aux_data_ps), err.(aux_data_ps), fmt="s", mfc="none", label="pseudoscalar")
        errorbar(xx, value.(aux_data_vec), err.(aux_data_vec), fmt="s", mfc="none", label="vector")
        fill_between(plat_ps[1]+62:plat_ps[2+62], v-e, v+e, alpha=0.6 )
        fill_between(plat_vec[1]+62:plat_vec[2]+62, v_vec-e_vec, v_vec+e_vec, alpha=0.6 )
        xlabel(L"$x_0/a$", size=18)
        ylabel(L"$af$", size=18)
        title("$(ensembles(dec0_ps[i])[1]) " * L"\mu_1="*"$(mu_list[i][1]) " *L" \mu_2="*"$(mu_list[i][2]) " , fontsize=18)
        ylim(v*0.95, v_vec*1.1)
        legend()
        display(gcf())
        t = "$(ensembles(dec0_ps[i])[1])_dec_"*"mu_1=$(mu_list[i][1]) " *"_mu_2=$(mu_list[i][2]) "*".pdf"
        savefig(joinpath(path_plot,t))
        close()
    end
    dec0_vec   .*=  [za(ensinfo.beta) for i in 1:length(dec0_vec)]
    

end

##

if sector["pseudo"]
    corr_pp    = get_corr(ensinfo, "G5", "G5", rw=rwf)
    corr_a1a1  = get_corr(ensinfo, "G1G5", "G1G5", rw=rwf)
    mu_list    = getfield.(corr_pp, :mu)
    l, s, muh  = get_mu(mu_list, ensinfo.deg)
    muc        = [zm_tm(ensinfo.beta) * muh[i] * sqrt(8*t0(ensinfo.beta)) for i in 1:length(muh)]
    matinfo_pp = comp_mat(ensinfo, corr_pp, tau)
    en0_ps     =  gevp_mass(matinfo_pp, t0=3, pl=true)
    dec0_ps    = gevp_dec(matinfo_pp, en0_ps, t0=3, n=1, pl=true)
    uwerr.(en0_ps)
    uwerr.(dec0_ps)
end
if sector["vector"]
    corr_a1a1    =  get_corr(ensinfo, "G1G5", "G1G5", rw=rwf)
    matinfo_a1a1 =  comp_mat(ensinfo, corr_a1a1, tau)
    en0_vec      =  gevp_mass(matinfo_a1a1, t0=3)
    uwerr.(en0_vec) 
    close("all") 
    dec0_vec     =  gevp_dec(matinfo_a1a1, en0_vec, t0=3, n=1, pl=true, pseudo=false)
    dec0_vec   .*=  [za(ensinfo.beta) for i in 1:length(dec0_vec)]
    uwerr.(dec0_vec)
    close("all")
end



## storing in BDIO
#save_bdio_obs(muc, "charm_mass", ensinfo)
if sector["masses"] # psuedo 
    ll_m = get_ll(mu_list, en0_ps, ensinfo.deg)
    ls_m = get_ls(mu_list, en0_ps, ensinfo.deg)
    ss_m = get_ss(mu_list, en0_ps, ensinfo.deg)
    lh_m = get_lh(mu_list, en0_ps, ensinfo.deg)
    sh_m = get_sh(mu_list, en0_ps, ensinfo.deg)
    hh_m = get_hh(mu_list, en0_ps, ensinfo.deg)
    en_ordered = vcat(ll_m, ls_m, ss_m, lh_m, sh_m, hh_m)
    ll_f = get_ll(mu_list, dec0_ps, ensinfo.deg)
    ls_f = get_ls(mu_list, dec0_ps, ensinfo.deg)
    ss_f = get_ss(mu_list, dec0_ps, ensinfo.deg)
    lh_f = get_lh(mu_list, dec0_ps, ensinfo.deg)
    sh_f = get_sh(mu_list, dec0_ps, ensinfo.deg)
    hh_f = get_hh(mu_list, dec0_ps, ensinfo.deg)
    f_ordered = vcat(ll_f, ls_f, ss_f, lh_f, sh_f, hh_f)
    #save_bdio_obs(en_ordered, "ps_masses", ensinfo)
    #save_bdio_obs(f_ordered, "ps_decays", ensinfo)
end
if sector["decays"] #vector 

    ll_m = get_ll(mu_list, en0_vec, ensinfo.deg)
    ls_m = get_ls(mu_list, en0_vec, ensinfo.deg)
    ss_m = get_ss(mu_list, en0_vec, ensinfo.deg)
    lh_m = get_lh(mu_list, en0_vec, ensinfo.deg)
    sh_m = get_sh(mu_list, en0_vec, ensinfo.deg)
    hh_m = get_hh(mu_list, en0_vec, ensinfo.deg)
    en_ordered = vcat(ll_m, ls_m, ss_m, lh_m, sh_m, hh_m)
    ll_f = get_ll(mu_list, dec0_vec, ensinfo.deg)
    ls_f = get_ls(mu_list, dec0_vec, ensinfo.deg)
    ss_f = get_ss(mu_list, dec0_vec, ensinfo.deg)
    lh_f = get_lh(mu_list, dec0_vec, ensinfo.deg)
    sh_f = get_sh(mu_list, dec0_vec, ensinfo.deg)
    hh_f = get_hh(mu_list, dec0_vec, ensinfo.deg)
    f_ordered = vcat(ll_f, ls_f, ss_f, lh_f, sh_f, hh_f)
    save_bdio_obs(en_ordered, "vec_masses", ensinfo)
    save_bdio_obs(f_ordered, "vec_decays", ensinfo)
end

##
aa = read_BDIO(joinpath(path_results,"vec_decays"), "N203")
## check that the cuts in cnfg are consistents with dm and t0
path_dm = "/Users/ale/Desktop/data"
dm = read_BDIO(path_dm, "delta")

t0_shifted =  read_BDIO(path_dm, "t0")

##
function save_bdio_obs(obs::Vector{uwreal}, final_path::String, ens::EnsInfo)
    cd(joinpath(path_results,"$(final_path)"))
    t = joinpath(pwd(),"$(ens.id)"*"_$(final_path).bdio")
    uinfo=0
    fb = BDIO_open(t,"w")
    for val in obs
        write_uwreal(val, fb, uinfo)
        uinfo +=1
    end
    BDIO_close!(fb)
end









######## TEST GEVP ############
#matinfo_pp = comp_mat(ensinfo, corr_pp, 4)

corr_pp    =  get_corr(ensinfo, "G5", "G5", rw=rwf)
corr_pa0   =  get_corr(ensinfo, "G5", "G1G5", rw=rwf)
corr_a0a0  =  get_corr(ensinfo, "G1G5", "G1G5", rw=rwf)
corr_a2a2  =  get_corr(ensinfo, "G2G5", "G2G5", rw=rwf)
corr_a3a3  =  get_corr(ensinfo, "G3G5", "G3G5", rw=rwf)



matinfo_multigamma = comp_mat_multigamma(ensinfo, corr_a0a0, corr_a2a2, corr_a3a3)


##
y0s = getfield.(matinfo_multigamma, :y0) .+1
mu_list = getfield.(matinfo_multigamma, :mu) 
m_fit = Vector{uwreal}(undef, length(matinfo_multigamma))
m_fit_vec = Vector{uwreal}(undef, length(matinfo_multigamma))

m_gevp = Vector{uwreal}(undef, length(matinfo_multigamma))
m_gevp_vec = Vector{uwreal}(undef, length(matinfo_multigamma))

for i in 1:length(matinfo_multigamma)
    mat = getfield(matinfo_multigamma[i], :mat_list) #matrices to diagonalise for a given sector [i]
    evals = getall_eigvals(mat, 2 )
    println(evals)
    plat = select_plateau(matinfo_multigamma[i].ensinfo, mu_list, ensinfo.deg)[i] .- y0s[i] 
    en = energies(evals)
    m_gevp[i] = plat_av(en[1], plat)
    m_gevp_vec[i] = plat_av(en[2], plat)
    #@. model(x,p) = p[1]*exp(-p[2]*x) + p[3]*exp(-p[4]*x)
    #tcut = 8
    #xx = collect(tcut:length(proj_pp)-10)
    #fit ,_ = fit_routine(model, xx, proj_pp[tcut:end-10], 4 )
    #_, idx = findmin(value.([fit[2],fit[4]]))
    #m_gevp[i] = fit[idx]
    #m_gevp[i], m_gevp_data = meff(proj_pp, plat.+1, pl=false, data=true)
    m_fit[i], m_fit_data = meff(corr_pp[i].obs[y0s[i]+1:end-1], plat, pl=false, data=true)
    m_fit_vec[i] = meff(corr_a0a0[i].obs[y0s[i]+1:end-1], plat, pl=false, data=false)
    uwerr(m_gevp[i])
    uwerr(m_gevp_vec[i])

    uwerr.(en[1])
    uwerr.(en[2])

    uwerr(m_fit[i])
    uwerr(m_fit_vec[i])

    #uwerr.(m_fit_data)
    xdata = collect(1:length(en[1]))
    #xdata = collect(1:length(m_gevp_data))
    #xdata1 = collect(1:length(m_fit_data))
    errorbar(xdata, value.(en[1]), yerr=err.(en[1]), fmt="^", label="en0 gevp")
    errorbar(xdata, value.(en[2]), err.(en[2]), fmt="^", label="en1 gevp")
    #errorbar(xdata, value.(m_gevp_data), err.(m_gevp_data), fmt="^", label="en0 gevp")
    #errorbar(xdata1, value.(m_fit_data), err.(m_fit_data), fmt="^", label="en0 fit")
    #fill_between(plat[1]:plat[2], value(m_fit[i])-err(m_fit[i]), value(m_fit[i])+err(m_fit[i]))
    #legend()
    ylim(0.1, 1.0)
    display(gcf())
    close("all")
    #gevp_mass = juobs.plat_av(en[1], [50,])

end
##
en0 = gevp_mass(matinfo_pp, t0=2)