
@doc raw"""
read_data(ens::String, g1::String="G5", g2::String="G5")
Given the ensemble id and the Dirac structure, this method reads the data in path_data and
returns a CData object. Replicas are automatically detected and loaded if present.
"""
function read_data(ens::String, g1::String="G5", g2::String="G5"; legacy::Bool=false)
    path = joinpath(path_data, ens)
    rep = filter(x->occursin("mesons.dat", x), readdir(path, join=true))
    if length(rep)!=0    
        length(rep)==1 ? (return read_mesons(rep[1], g1, g2, id=ens, legacy=legacy)) : (return read_mesons(rep, g1, g2, id=ens, legacy=legacy))
    else
        error("mesons.dat file not found for ensemble ", ens, " in path ", path)
    end
end
function read_rw(ens::String)
    rep = filter(x->occursin(ens, x), readdir(path_rw, join=true))
    if length(rep)!=0
        try
            length(rep) == 1 ? (return read_ms1(rep[1])) : (return read_ms1.(rep)) 
        catch
            length(rep) == 1 && length(rep)!=0 ? (return read_ms1(rep[1], v="1.4")) : (return read_ms1.(rep, v="1.4")) 
        end
    else
        error("ms1.dat file not found for ensemble ", ens, " in path ", path_rw)
    end
end
function read_mass_dev(ens::String)
    rep = filter(x->occursin(ens,x), readdir(path_md, join=true))
    try
        if length(rep) == 1
            return [read_md(rep[1])]
        else
            return [read_md(rep[1]), read_md(rep[2])]
        end
    catch
        error("pbp.dat file not found for ensemble ", ens, " in path ", path_md)
    end
end
function read_t0(ens::String)
path = joinpath(path_data, ens)
rep = filter(x->occursin("ms.dat", x), readdir(path, join=true))
if length(rep)!=0
    length(rep) == 1 ? (return read_ms(rep[1])) : (return read_ms.(rep)) 
else
    error("ms.dat file not found for ensemble ", ens, " in path ", path)
end
end
function read_BDIO(path::String, ens_id::String)
    file = filter(x->occursin(ens_id, x), readdir(path, join=true))
    r = Vector{uwreal}(undef, 0)
    fb = BDIO_open(file[1], "r")
    BDIO_seek!(fb)
    push!(r, read_uwreal(fb))
    while BDIO_seek!(fb, 2)
        push!(r, read_uwreal(fb))
    end
    BDIO_close!(fb)
    return r
end
function save_bdio_obs(obs::Vector{uwreal}, final_path::String, ens::EnsInfo)
    cd(joinpath(path_results,"$(final_path)"))
    t = joinpath(pwd(),"$(ens.id)"*"_$(final_path).bdio")
    uinfo=0
    fb = BDIO_open(t,"w")
    for val in obs
        write_uwreal(val, fb, uinfo)
        uinfo +=1
    end
    BDIO_close!(fb)
end