function get_corr(ens::EnsInfo, g1::String="G5", g2::String="G5"; rw::Bool=false, legacy::Bool=false)
    aux = read_data(ens.id, g1, g2, legacy=legacy)
    truncate_data!(aux, truncate[ens.id])
    #if ens.id =="J303"
    #    truncate_data!(aux, 739)
    #end
    #if ens.id =="H101"
    #    #truncate_data!(aux, [878, 999])
    #end
    !rw ? obs = corr_obs.(aux, L=ens.L) : obs = corr_obs.(aux, L=ens.L, rw=read_rw(ens.id))
    return obs
end
function get_t0(ens::EnsInfo; rw::Bool=false, pl::Bool=false)
    data = read_t0(ens.id)
    plat = select_plateau(ens) 
    !rw ? t0 = comp_t0(data, plat, L=ens.L, pl=pl) : t0 = comp_t0(data, plat, L=ens.L, pl=pl, rw=read_rw(ens.id))
    return t0
end
function get_mu(mu_list::Vector{Vector{Float64}}, deg::Bool)
    mu_sorted = unique(sort(minimum.(mu_list)))

    mul = mu_sorted[1]
    deg ? mus = 0.0 : mus = mu_sorted[2]
    muh = unique(maximum.(mu_list))
    muh = filter(x-> x > mul && x > mus, muh)

    return mul, mus, muh
end
function get_ll(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_ll = Vector{uwreal}(undef, 0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] == mu[2] #l-l
            push!(obs_ll, obs[k])
        end
    end
    return obs_ll
end
function get_ls(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_ls = Vector{uwreal}(undef, 0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mus in mu  #l-l
            push!(obs_ls, obs[k])
        end
    end
    return obs_ls
end
function get_lh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_lh = Vector{uwreal}(undef, 0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] != mu[2] && !(mus in mu) #l-h
            push!(obs_lh, obs[k])
        end
    end
    return obs_lh
end
function get_ss(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_ss = Vector{uwreal}(undef, 0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mus in mu && mu[1] == mu[2] #s-s
            push!(obs_ss, obs[k])
        end
    end
    return obs_ss
end
function get_sh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_sh = Vector{uwreal}(undef, 0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mus in mu && mu[1] != mu[2] && !(mul in mu)#s-h
            push!(obs_sh, obs[k])
        end
    end
    return obs_sh
end
function get_hh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_hh = Vector{uwreal}(undef,0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        for i =1:length(muh)
            if muh[i] in mu && mu[1] == mu[2]#h-h
                push!(obs_hh, obs[k])
            end
        end
    end
    return obs_hh
end
function comp_mat(ensinfo::EnsInfo, tot_obs::Vector{juobs.Corr}, tau::Int64)
    mat_info = Vector{MatInfo}(undef, length(tot_obs))
    for i = 1:length(tot_obs)
        a11 = tot_obs[i].obs[1:end-1-2*tau]
        a12 = tot_obs[i].obs[1+tau:end-1-tau]
        a22 = tot_obs[i].obs[1+2*tau:end-1]
        aux = juobs.get_matrix([a11, a22], [a12])
        mat_info[i] = MatInfo(ensinfo, aux, tot_obs[i].mu, tot_obs[i].y0)
    end
    return mat_info
end
function comp_mat_multigamma(ensinfo::EnsInfo,tot11::Vector{juobs.Corr}, tot12::Vector{juobs.Corr}, tot22::Vector{juobs.Corr})
    mu = getfield.(tot11,:mu)
    y0 = getfield.(tot11,:y0) .+1

    mat_info = Vector{MatInfo}(undef, length(mu))
    for i = 1:length(mu)
        a11 = tot11[i].obs[y0[i]:end-1]
        a12 = tot12[i].obs[y0[i]:end-1]
        a22 = tot22[i].obs[y0[i]:end-1]
        #res[i] = infoMatt(a11, a12, a22)
        aux = juobs.get_matrix([a11, a22], [a12])
        mat_info[i] = MatInfo(ensinfo, aux, mu[i], y0[i])
    end
    return mat_info
end
function comp_mat_lukas(ensinfo::EnsInfo, tot_obs::Vector{juobs.Corr}, dim::Int64)
    mat_info = Vector{MatInfo}(undef, length(tot_obs))
    for i in 1:length(tot_obs)
        a11 = tot_obs[i].obs[1+1:end-6]
        a22 = tot_obs[i].obs[3+1:end-4]
        a33 = tot_obs[i].obs[5+1:end-2]
        a12 = tot_obs[i].obs[2+1:end-5]
        a13 = tot_obs[i].obs[3+1:end-4]
        a23 = tot_obs[i].obs[4+1:end-3]
        aux = juobs.get_matrix([a11, a22, a33], [a12, a13, a23])
        mat_info[i] = MatInfo(ensinfo, aux, tot_obs[i].mu, tot_obs[i].y0)        
    end
    return mat_info
end
function gevp_mass(mat_obs::Vector{MatInfo}; t0::Int64=2,pl::Bool=true, wpm::Union{Nothing, Dict{Int64,Vector{Float64}}}=nothing)
    y0s = getfield.(mat_obs, :y0) 
    mu_list = getfield.(mat_obs, :mu) 
    plat_en0 = Vector{uwreal}(undef, length(mat_obs))
    plat_en1 = Vector{uwreal}(undef, length(mat_obs))
    for i in 1:length(mat_obs)
        mat = getfield(mat_obs[i], :mat_list)[y0s[i]+1:end-1] #matrices to diagonalise for a given sector [i]
        evals = getall_eigvals(mat, t0)
        en = energies(evals, wpm=wpm) #ground and excited state energy
        plat = select_plateau(mat_obs[i].ensinfo, mu_list, path_plat_meff)[i] .- y0s[i] 
        println(plat)
        plat_en0[i] = plat_av(en[1], plat)
        if pl 
            uwerr(plat_en0[i])
            xx = collect(1:length(en[1]))
            v = value(plat_en0[i])
            e = err(plat_en0[i])
            errorbar(xx, value.(en[1]), err.(en[1]), fmt="s")
            fill_between(plat[1]:plat[2], v-e, v+e )
            ylim(v*0.98, v*1.02)
            display(gcf())
            close()
        end
    end
    return plat_en0
end
function gevp_mass(mat_obs::Vector{MatInfo}, path_m_ps::String, path_m_vec::String; t0::Int64=2,pl::Bool=true, wpm::Union{Nothing, Dict{Int64,Vector{Float64}}}=nothing)
    y0s = getfield.(mat_obs, :y0) 
    mu_list = getfield.(mat_obs, :mu) 
    plat_en0 = Vector{uwreal}(undef, length(mat_obs))
    plat_en1 = Vector{uwreal}(undef, length(mat_obs))
    for i in 1:length(mat_obs)
        mat = getfield(mat_obs[i], :mat_list) #matrices to diagonalise for a given sector [i]
        evals = getall_eigvals(mat, t0)
        en = energies(evals, wpm=wpm) #ground and excited state energy
        plat_ps = select_plateau(mat_obs[i].ensinfo, mu_list, path_m_ps)[i] .- y0s[i] 
        println(plat_ps)
        plat_vec = select_plateau(mat_obs[i].ensinfo, mu_list, path_m_vec)[i] .- y0s[i] 
        println(plat_vec)
        plat_en0[i] = plat_av(en[1], plat_ps)
        plat_en1[i] = plat_av(en[2], plat_vec)

        if pl 
            uwerr(plat_en0[i])
            uwerr(plat_en1[i])
            xx    = collect(1:length(en[1]))
            v     = value(plat_en0[i])
            e     = err(plat_en0[i])
            v_vec = value(plat_en1[i])
            e_vec = err(plat_en1[i])
            errorbar(xx, value.(en[1]), err.(en[1]), fmt="s", mfc="none", label="pseudoscalar")
            fill_between(plat_ps[1]:plat_ps[2], v-e, v+e, alpha=0.6 )
            errorbar(xx, value.(en[2]), err.(en[2]), fmt="s",mfc="none", label="vector")
            fill_between(plat_vec[1]:plat_vec[2], v_vec-e_vec, v_vec+e_vec, alpha=0.6 )
            xlabel(L"$x_0/a$", size=18)
            ylabel(L"$am_{\mathrm{eff}}$", size=18)
            title("$(ensembles(plat_en0[i])[1]) " * L"\mu_1="*"$(mu_list[i][1]) " *L" \mu_2="*"$(mu_list[i][2]) " )
            ylim(v*0.95, v_vec*1.05)
            legend()
            display(gcf())
            t = "$(ensembles(plat_en0[i])[1])_m_eff_"*"mu_1=$(mu_list[i][1]) " *"_mu_2=$(mu_list[i][2]) "*".pdf"
            #savefig(joinpath(path_plot,t))
            close()
        end
    end
    return plat_en0, plat_en1
end

function gevp_dec(mat_obs::Vector{MatInfo}, mass::Vector{uwreal}, path_plat::String; t0::Int64=2, wpm::Union{Nothing, Dict{Int64,Vector{Float64}}}=nothing, pl::Bool=true, n::Int64=1, pseudo::Bool=true, wilson::Bool=false)
    y0s       = getfield.(mat_obs, :y0)
    mu_list   = getfield.(mat_obs, :mu) 
    plat_dec0 = Vector{uwreal}(undef, length(mat_obs))
    data      = Vector{Vector{uwreal}}(undef,0)
    for i in 1:length(mat_obs)
        mat   = getfield(mat_obs[i], :mat_list)[y0s[i]+2:end-1] #matrices to diagonalise for a given sector [i]
        evecs = getall_eig(mat, t0)
        elem  = mat_elem(evecs, mat, mass[i], n) #ground and excited state energy
        push!(data, elem)
        plat  = select_plateau(mat_obs[i].ensinfo, mu_list, path_plat)[i] .- y0s[i]
        try
            if pseudo
                plat_dec0[i] = dec(elem, plat, mass[i], mu_list[i], pl=pl, wilson=wilson)
            else
                plat_dec0[i] = vec_dec(elem, plat, mass[i], pl=pl)
            end
        catch
            println("Decay constant for the ensemble ", mat_obs[i].ensinfo.id, " failed in the sector ", mu_list[i], " pseudo sector?", pseudo)
            println("The associated effective mass is ", mass[i])
            plat_dec0[i] = uwreal(0)
        end
    end
    return plat_dec0, data
end

function getall_eig(a::Vector{Matrix{uwreal}}, t0; iter=10)
    n = length(a)
    res = Vector{Matrix{uwreal}}(undef, n)
    [res[i] = uweigvecs(a[i], a[t0]) for i=1:n]
    return res 
end
function mat_elem(evec::Vector{Matrix{uwreal}}, ct_mat::Vector{Matrix{uwreal}}, mass::uwreal,  n::Int64)
    t = length(evec)
    res = Vector{uwreal}(undef, t)
    for i in 1:t
        aux = uwdot(evec[i][:,n], uwdot(ct_mat[i], evec[i][:,n]))[1]
        aux2 = 1 / (aux^2)^(0.25)   * exp(mass * (i)/2)
        res[i] = aux2 * uwdot(evec[i][:,n], ct_mat[i][:,n])
    end
    return res
end
function dec(mat_elem::Vector{uwreal}, plat::Array{Int64}, mass::uwreal, mu::Vector{Float64}; pl::Bool=true, wilson::Bool=false)
    aux = plat_av(mat_elem, plat)
    if pl
        uwerr(aux)
        x = 1:length(mat_elem)
        y = value.(mat_elem)
        dy = err.(mat_elem)
        v = value(aux)
        e = err(aux)
        figure()
        errorbar(x, y, yerr=dy, fmt="*", color="red")
        fill_between(plat[1]:plat[2], v-e, v+e, color="green")    
        ylim(v - 10*e, v + 10*e)
        display(gcf())
        close()
    end
    if !wilson
        return sqrt(2/ (mass)^3) * sum(mu)* aux
    else
        return sqrt(2/mass) * aux 
    end
end
function vec_dec(mat_elem::Vector{uwreal}, plat::Array{Int64}, mass::uwreal; pl::Bool=true)
    aux = plat_av(mat_elem, plat)
    if pl
        uwerr(aux)
        x = 1:length(mat_elem)
        y = value.(mat_elem)
        dy = err.(mat_elem)
        v = value(aux)
        e = err(aux)
        figure()
        ylim(v - 10*e, v + 10*e)
        errorbar(x, y, yerr=dy, fmt="*", color="red")
        fill_between(plat[1]:plat[2], v-e, v+e, color="green")    
        display(gcf())
    end
    return sqrt(2 / mass) * aux #aux / mass
end
function param_av(fit_masses::Vector{uwreal})
    w = 1 ./ err.(fit_masses).^2
    av = sum(w .* fit_masses) / sum(w)
    return av  
end
function get_meff(obs::Vector{juobs.Corr}, ens::EnsInfo; pl::Bool=false, wpm::Union{Nothing, Dict{String,Vector{Float64}}}=nothing)
    mu_list  = getfield.(obs, :mu)
    plat = select_plateau(ens, mu_list)
    return meff.(obs, plat, pl=pl, wpm=wpm)
end
function get_f(obs::Vector{juobs.Corr}, m::Vector{uwreal}, ens::EnsInfo; pl::Bool=false)
    mu_list = getfield.(obs, :mu)
    plat = select_plateau(ens, mu_list)
    return dec_const_pcvc.(obs, plat, m, pl=pl)
end
function select_plateau(ensinfo::EnsInfo)
    ens = ensinfo.id
    f = readdlm(path_plat_meff)
    head  = String.(f[:,1])
    delim = findall(x->occursin("#", x), head)
    edelim = findfirst(x->occursin(ens, x), head)
    cdelim = findfirst(x-> x.== edelim, delim)
    del = delim[cdelim]+1 : delim[cdelim+1]-1
    head_ = String.(f[del, 1])
    plat_ = Int64.(f[del, 2:3])
    plat = Vector{Int64}(undef, 2)
    n = findfirst(x-> occursin("t0", x), head_)
    plat[1] = plat_[n,1]
    plat[2] = plat_[n,2]
    return plat
end
function select_plateau(ensinf::EnsInfo, mu_list, path_plat)
    ens =ensinf.id
    deg = ensinf.deg
    mul, mus, muh = get_mu(mu_list, deg)
    f = readdlm(path_plat)
    head = String.(f[:, 1])

    delim = findall(x-> occursin("#", x ), head)
    edelim = findfirst(x-> occursin(ens, x), head)
    cdelim = findfirst(x-> x.== edelim, delim)

    del = delim[cdelim]+1 : delim[cdelim+1]-1
    #del = delim
    plat = Vector{Vector{Int64}}(undef, 0)
    #plat = Dict()
    plat_ = Int64.(f[del, 2:3])
    head_ = String.(f[del, 1])
    
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] != mu[2] && !(mus in mu)  #heavy-light
            n = findfirst(x-> occursin("lh", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]]  ) 
        elseif mul in mu && mu[1] == mu[2] #light-light
            n = findfirst(x-> occursin("ll", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )               
        elseif mul in mu && mus in mu#strange-light
            n = findfirst(x-> occursin("ls", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] != mu[2] && !(mul in mu)#heavy-strange
            n = findfirst(x-> occursin("sh", x ), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] == mu[2] && !(mul in mu)#strange-strange
            n = findfirst(x-> occursin("ss", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif !(mul in mu) && !(mus in mu) #heavy-heavy
            n = findfirst(x-> occursin("hh", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        end
    end
    return plat
end

function match_muc(muh, m_lh, m_sh, target)
    M = (2/3 .* m_lh .+ 1/3 .* m_sh) 
    par, chi2exp = lin_fit(muh, M)
    muh_target = x_lin_fit(par, target)
    return muh_target
end
function match_muc_heavymass(muh, m_hh, target)
    par, chi2exp = lin_fit(muh, m_hh)
    muh_target = x_lin_fit(par, target)
    return muh_target
end
function match_phih(muh,phih, phih_ph)
    par, chi2exp = lin_fit(muh, phih)
    muh_target = x_lin_fit(par, phih_ph)
    
    return muh_target
end

function aic_weigth(cat::Cat; cut::Bool=false)
    w_aux = exp.(-1/2 * (getfield(cat, :aic)))
    w = w_aux ./ sum(w_aux)
    return w
end
function aic_syst(cat::Cat)
    w  = aic_weigth(cat)
    aux1 = sum( w .* getfield(cat,:fit_res).^2)
    aux2 = sum( w .* getfield(cat,:fit_res))^2
    return sqrt(abs(value(aux1 - aux2)))
end
function aic_model_ave(cat::Cat)
    w  = aic_weigth(cat)
    res = sum( w.* getfield(cat, :fit_res))
    uwerr(res)
    return res
end
function category_av(cat_arr::Vector{Cat})
    best_res = aic_model_ave.(cat_arr)
    println(best_res)
    syst = aic_syst.(cat_arr)
    tot_err = sqrt.(err.(best_res).^2 .+ syst.^2)
    println(tot_err)
    w_aux = 1 ./ (tot_err)
    w = w_aux ./ sum(w_aux)
    println(w)
    aux1 = sum( w .* best_res.^2)
    aux2 = sum( w .* best_res)^2
    av = sum( w .* best_res) 
    return av, sqrt(abs(value(aux1 - aux2)))
end

function syst_av(syst_err::Vector{Float64})
    aux = sum(syst_err.^2)
    return sqrt(aux)
end

function fit_over_cat!(cat_tot::Vector{Cat}, models::Vector{Vector{Function}}, phi2_phys::uwreal, model_param::Vector{Int64} ; decay::Bool=false, weight_dof::Int64=2, xycovar::Bool=false, wpm::Union{Dict{Int64,Vector{Float64}},Dict{String,Vector{Float64}}, Nothing}=nothing)
    models      = vcat(models...)

    function continuum_dependece(params, phi2, phih; decay_phih_dep::Bool=decay)
        if !decay_phih_dep
            return Mrat * (params[1] + params[2]*phi2 + params[3]*phih)
        else
            return params[1] + params[2]*phi2 + params[3]/ sqrt(phih)
        end
    end

    for cat in cat_tot
        println("\n", cat.info, "\n")
        for (i, mod) in enumerate(models)
            if !xycovar
                fit_params, chi2_vs_chi2exp = juobs.fit_routine(mod, value.(cat.x_tofit), cat.obs, model_param[i], wpm=wpm)
            else
                fit_params, chi2_vs_chi2exp = juobs.fit_routine(mod, cat.x_tofit, cat.obs, model_param[i], covar=true, wpm=wpm)                
            end
            println(fit_params, "\n")
            println(phi2_phys, "\n")
            println(cat.phih_ph, "\n")

            val_mod_i = continuum_dependece(fit_params, phi2_phys, cat.phih_ph)
            push!(cat.dof, length(cat.obs)-model_param[i] ) 
            AIC = chi2_vs_chi2exp * cat.dof[i] + weight_dof * model_param[i]
            
            tmp  = [val_mod_i, fit_params, chi2_vs_chi2exp, model_param[i], AIC]
            symb = [:fit_res, :fit_param, :chi2_vs_exp, :n_param, :aic]
            for (k,ss) in enumerate(symb)
                push!(getfield(cat, ss), tmp[k] )
            end
        end
    end
end