function plot_chi(c::Cat, ff::Vector{Vector{Function}}, ylab::LaTeXString, phi2_ph::uwreal, ensinfo::Vector{EnsInfo}; p::Union{String, Nothing}=nothing,
    wpm::Union{Dict{String,Vector{Float64}}, Nothing}=nothing, n::Union{Nothing, Int64}=nothing)
    phi2_sym = 0.744666
    color = ["orange", "green", "blue", "darkviolet"]
    fmt = ["^", "v", "<", ">"]
    if isnothing(n)
        aic, n = findmin(getfield(c, :aic))
    end
    par_n = getfield(c, :n_param)[n]
    upar = getfield(c, :fit_param)[n]
    res = getfield(c, :fit_res)[n]

    func = vcat(ff...)[n]
    xx = [fill(0.0, 100) Float64.(range(0.01, phi2_sym, length=100)) fill(c.phih_ph,100)]
    yy = func(xx, upar)
    p2 = unique(c.x_tofit[:,2])
    idx_a2 = findfirst.(isequal.(unique(value.(c.x_tofit[:,1]))), [value.(c.x_tofit[:,1])])
    a2 = c.x_tofit[:,1][idx_a2]
    println(a2)
    println("\n")
    println(p2)
    x = [a2 p2 fill(c.phih_ph, length(p2))]
    x = [a2 p2 c.x_tofit[1:8, 3]]
    x2 = [a2 p2 fill(c.phih_ph, 8)]
    y = c.obs[1:8] - func(x, upar) + func(x2, upar)
    if isnothing(wpm)
        uwerr.(yy)
        uwerr(res)
        uwerr.(y)
    else
        [uwerr(yy_aux, wpm) for yy_aux in yy]
        uwerr(res, wpm)
        [uwerr(y_aux, wpm) for y_aux in y]
    end

    figure()
    fill_between(xx[:,2], value.(yy)+ err.(yy), value.(yy)-err.(yy), alpha=0.3, color="red")
    errorbar(value(phi2_ph), value(res), err(res), fmt="s", color="red", label=L"$\phi_2 = \phi^\mathrm{phys}_2$")
    for (k,b) in enumerate(sort(unique(getfield.(ensinfo, :beta))))
        n_ = findall(x->x.beta == b, ensinfo)
        a2_aux = mean(value.(a2[n_]))
        errorbar(value.(x[n_,2]), value.(y[n_]), err.(y[n_]), fmt=fmt[k], label=string(L"$\beta =$", b), color=color[k])
        xxx = [fill(a2_aux, 100) Float64.(range(0.0, phi2_sym, length=100)) fill(value(c.phih_ph), 100)]
        yyy = func(xxx, upar)
        plot(xxx[:,2], value.(yyy), ls="--", color=color[k])
    end
    ylabel(ylab)
    xlabel(L"$\phi_2$")
    legend()
    tight_layout()
    if !isnothing(p)
        aux1, aux2 = split(c.info, ", ")
        t = string("chi_", aux1, "_", aux2, ".pdf")
        savefig(joinpath(p, t))
        close()
    else
        display(gcf())
    end

end

function plot_cont(c::Cat, ff::Vector{Vector{Function}}, ylab::LaTeXString, phi2_ph::uwreal, ensinfo::Vector{EnsInfo}; p::Union{String, Nothing}=nothing,
wpm::Union{Dict{String,Vector{Float64}}, Nothing}=nothing, n::Union{Nothing, Int64}=nothing)
    if isnothing(n)
        aic, n = findmin(getfield(c, :aic))
    end
    par_n = getfield(c, :n_param)[n]
    upar = getfield(c, :fit_param)[n]
    res = getfield(c, :fit_res)[n]

    func = vcat(ff...)[n]
    xx = [Float64.(range(0, 0.05, length=100)) fill(phi2_ph, 100) fill(c.phih_ph,100)]
    yy = func(xx, upar)


    if isnothing(wpm)
        uwerr(res)
        uwerr.(yy)
    else
        [uwerr(yy_aux, wpm) for yy_aux in yy]
        uwerr(res, wpm)
    end


    v = value.(yy)
    e = err.(yy)
    figure()
    xlabel(L"$\frac{a^2}{8t_0}$")
    ylabel(ylab)
    fill_between(xx[:, 1], v+e, v-e, color="red", alpha=0.3)
    errorbar(0.0, value(res), err(res), fmt="s", color="red", label="Continuum")

    for b in unique(getfield.(ensinfo, :beta))
        n_ = findall(x->x.beta == b, ensinfo)
        a2 = c.x_tofit[n_, 1]
        pc = c.x_tofit[n_, 3]
        a2 = fill(mean(a2), length(pc))
        x = [a2 fill(phi2_ph, length(a2)) pc]
        x2 = [a2 fill(phi2_ph, length(a2)) fill(c.phih_ph, length(a2))]
        y = c.obs[n_] - func(x, upar) + func(x2, upar)
        #y = func(x,upar)
        isnothing(wpm) ? uwerr.(y) : [uwerr(y_, wpm) for y_ in y]
        errorbar(value.(a2), value.(y), err.(y), fmt="x", color="black")
    end
    legend()
    tight_layout()
    #errorbar(value.(x[:,1]), value.(y), err.(y), fmt="x", color="black")

    if !isnothing(p)
        aux1, aux2 = split(c.info, ", ")
        t = string("sca_", aux1, "_", aux2, ".pdf")
        savefig(joinpath(p, t))
        close()
    else
        display(gcf())
    end

end


import Base: +,*,/
#uwreal +
function +(uw::uwreal, vec::Vector{uwreal})
    n = size(vec,1)
    return fill(uw, n) .+ vec
end
function +(vec::Vector{uwreal}, uw::uwreal )  uw+vec  end
#uwreal *
function *(uw::uwreal, vec::Vector{uwreal})
    n = size(vec,1)
    return fill(uw, n) .* vec
end
function *(vec::Vector{uwreal}, uw::uwreal )  uw*vec  end
#any +
function +(uw::uwreal, vec::Vector{Any})
    n = size(vec,1)
    return fill(uw, n) .+ vec
end
function +(vec::Vector{Any}, uw::uwreal )  uw+vec  end
#any *
function *(uw::uwreal, vec::Vector{Any})
    n = size(vec,1)
    return fill(uw, n) .* vec
end
function *(vec::Vector{Any}, uw::uwreal )  uw*vec  end
#float +
function +(uw::uwreal, vec::Vector{Float64})
    n = size(vec,1)
    return fill(uw, n) .+ vec
end
function +(vec::Vector{Float64}, uw::uwreal )  uw+vec  end
#float *
function *(uw::uwreal, vec::Vector{Float64})
    n = size(vec,1)
    return fill(uw, n) .* vec
end
function *(vec::Vector{Float64}, uw::uwreal )  uw*vec  end
